@@ -22,32 +22,13 @@ import uuid
 from dateutil.parser import parser
 
 # external packages
-from PyQt5.QtCore import QObject, pyqtSignal
 import requests
 
 # local imports
+from base.driverDataClass import Signals
 
 
-class AlpacaSignals(QObject):
-
-    """
-    The AlpacaSignals class offers a list of signals to be used and instantiated by
-    the Alpaca class to get signals for triggers for finished tasks to
-    enable a gui to update their values transferred to the caller back.
-
-    This has to be done in a separate class as the signals have to be subclassed from
-    QObject.
-    """
-
-    __all__ = ['AlpacaSignals']
-
-    serverConnected = pyqtSignal()
-    serverDisconnected = pyqtSignal(object)
-    deviceConnected = pyqtSignal(str)
-    deviceDisconnected = pyqtSignal(str)
-
-
-class AlpacaBase:
+class AlpacaBase(Signals):
     """
     the class AlpacaBase inherits all information and handling of alpaca devices
     this class will be only referenced from other classes and will be not used
@@ -63,7 +44,7 @@ class AlpacaBase:
     def __init__(self):
         super().__init__()
 
-        self.signals = AlpacaSignals()
+        self.signals = Signals()
 
         self.protocol = 'http'
         self.host = ('localhost', 11111)
@@ -120,33 +101,31 @@ class AlpacaBase:
         )
 
         try:
-            response = requests.get(url, timeout=3)
+            response = requests.get(url, timeout=5)
 
         except requests.exceptions.Timeout:
             self.log.info('timeout')
-            return None
+            return []
 
         except requests.exceptions.ConnectionError:
             self.log.debug('[connection error')
-            return None
+            return []
 
         except Exception as e:
             self.log.critical(f'[error in request: {e}')
-            return None
+            return []
 
         if response.status_code == 400 or response.status_code == 500:
             self.log.debug(f'{response.text}')
-            return None
+            return []
 
         response = response.json()
-
         if response['ErrorNumber'] != 0:
             self.log.warning(f'{response} err:{response["ErrorNumber"]}'
                              f',{response["ErrorMessage"]}')
-            return None
+            return []
 
         self.log.trace(f'[response:{response}')
-
         return response['Value']
 
     def discoverDevices(self):
@@ -154,9 +133,6 @@ class AlpacaBase:
 
         :return:
         """
-
-        self.log.trace('discover devices')
-
         url = '{0}://{1}:{2}/management/v{3}/configureddevices'.format(
             self.protocol,
             self.host[0],
@@ -165,33 +141,32 @@ class AlpacaBase:
         )
 
         try:
-            response = requests.get(url, timeout=3)
+            response = requests.get(url, timeout=10)
 
         except requests.exceptions.Timeout:
             self.log.info('timeout')
-            return None
+            return []
 
         except requests.exceptions.ConnectionError:
             self.log.debug('[connection error')
-            return None
+            return []
 
         except Exception as e:
             self.log.critical(f'[error in request: {e}')
-            return None
+            return []
 
         if response.status_code == 400 or response.status_code == 500:
             self.log.debug(f'{response.text}')
-            return None
+            return []
 
         response = response.json()
 
         if response['ErrorNumber'] != 0:
             self.log.warning(f'{response} err:{response["ErrorNumber"]}'
                              f',{response["ErrorMessage"]}')
-            return None
+            return []
 
         self.log.trace(f'[response:{response}')
-
         return response['Value']
 
     def get(self, attr: str, **data):
@@ -205,42 +180,43 @@ class AlpacaBase:
             **data: Data to send with request.
 
         """
-
         if not self.deviceName:
-            return None
+            return []
 
         uid = uuid.uuid4().int % 2**32
         data['ClientTransactionID'] = uid
 
-        self.log.trace(f'[{uid:10d}] {self.baseUrl}, attr:[{attr}]')
+        self.log.trace(f'[{uid:10d}] {self.baseUrl}/{attr}], data:[{data}]')
 
         try:
-            response = requests.get(f'{self.baseUrl}/{attr}', data=data, timeout=5)
+            response = requests.get(f'{self.baseUrl}/{attr}', params=data, timeout=10)
 
         except requests.exceptions.Timeout:
             self.log.info(f'[{uid:10d}] timeout')
-            return None
+            return []
 
         except requests.exceptions.ConnectionError:
             self.log.debug(f'[{uid:10d}] connection error')
-            return None
+            return []
 
         except Exception as e:
             self.log.critical(f'[{uid:10d}] error in request: {e}')
-            return None
+            return []
 
         if response.status_code == 400 or response.status_code == 500:
             self.log.debug(f'{response.text}')
-            return None
+            return []
 
         response = response.json()
 
         if response['ErrorNumber'] != 0:
-            self.log.warning(f'{response} err:{response["ErrorNumber"]}'
-                             f',{response["ErrorMessage"]}')
-            return None
+            self.log.debug(f'{response} err:{response["ErrorNumber"]}'
+                           f',{response["ErrorMessage"]}')
+            return []
+
+        if attr != 'imagearray':
+            self.log.trace(f'[{uid:10d}] response:{response}')
 
-        self.log.trace(f'[{uid:10d}] response:{response}')
         return response['Value']
 
     def put(self, attr: str, **data):
@@ -255,7 +231,7 @@ class AlpacaBase:
 
         """
         if not self.deviceName:
-            return None
+            return []
 
         uid = uuid.uuid4().int % 2**32
         data['ClientTransactionID'] = uid
@@ -263,32 +239,31 @@ class AlpacaBase:
         self.log.trace(f'[{uid:08d}] {self.baseUrl}, attr:[{attr}]')
 
         try:
-            response = requests.put(f'{self.baseUrl}/{attr}', data=data, timeout=5)
+            response = requests.put(f'{self.baseUrl}/{attr}', data=data, timeout=10)
 
         except requests.exceptions.Timeout:
             self.log.info(f'[{uid:10d}] timeout')
-            return None
+            return []
 
         except requests.exceptions.ConnectionError:
             self.log.debug(f'[{uid:10d}] connection error')
-            return None
+            return []
 
         except Exception as e:
             self.log.critical(f'[{uid:10d}] Error in request: {e}')
-            return None
+            return []
 
         if response.status_code == 400 or response.status_code == 500:
             self.log.debug(f'[{uid:10d}] {response.text}')
-            return None
+            return []
 
         response = response.json()
 
         if response['ErrorNumber'] != 0:
             self.log.warning(f'err:{response["ErrorNumber"]},{response["ErrorMessage"]}')
-            return None
+            return []
 
         self.log.trace(f'[{uid:10d}] response:{response}')
-
         return response
 
     def action(self, Action: str, *Parameters):
@@ -367,7 +342,7 @@ class AlpacaBase:
         """
         val = self.get('driverinfo')
 
-        if val is None:
+        if not val:
             return ''
 
         return [i.strip() for i in val.split(',')]
@@ -1127,21 +1102,18 @@ class Camera(AlpacaBase):
         """
         Indicate that an image is ready to be downloaded.
         """
-
         return self.get("imageready")
 
     def ispulseguiding(self):
         """
         Indicatee that the camera is pulse guideing.
         """
-
         return self.get("ispulseguiding")
 
     def lastexposureduration(self):
         """
         Report the actual exposure duration in seconds (i.e. shutter open time).
         """
-
         return self.get("lastexposureduration")
 
     def lastexposurestarttime(self):
@@ -1153,28 +1125,24 @@ class Camera(AlpacaBase):
         :return:
             Start time of the last exposure in FITS standard format.
         """
-
         return self.get("lastexposurestarttime")
 
     def maxadu(self):
         """
         Camera's maximum ADU value.
         """
-
         return self.get("maxadu")
 
     def maxbinx(self):
         """
         Maximum binning for the camera X axis.
         """
-
         return self.get("maxbinx")
 
     def maxbiny(self):
         """
         Maximum binning for the camera Y axis.
         """
-
         return self.get("maxbiny")
 
     def numx(self, NumX=None):
@@ -1187,7 +1155,6 @@ class Camera(AlpacaBase):
         :return:
             Current subframe width.
         """
-
         if NumX is None:
             return self.get("numx")
 
@@ -1203,12 +1170,20 @@ class Camera(AlpacaBase):
         :return:
             Current subframe height.
         """
-
         if NumY is None:
             return self.get("numy")
 
         self.put("numy", NumY=NumY)
 
+    def offset(self, Offset=None):
+        """
+        Indicate the camera's Offset.
+        """
+        if Offset is None:
+            return self.get("offset")
+
+        self.put("offset", Offset=Offset)
+
     def percentcompleted(self):
         """
         Indicate percentage completeness of the current operation.
@@ -1218,28 +1193,24 @@ class Camera(AlpacaBase):
             completion).
 
         """
-
         return self.get("percentcompleted")
 
     def pixelsizex(self):
         """
         Width of CCD chip pixels (microns).
         """
-
         return self.get("pixelsizex")
 
     def pixelsizey(self):
         """
         Height of CCD chip pixels (microns).
         """
-
         return self.get("pixelsizey")
 
     def readoutmode(self, ReadoutMode=None):
         """
         Indicate the camera's readout mode as an index into the array ReadoutModes.
         """
-
         if ReadoutMode is None:
             return self.get("readoutmode")
 
@@ -1249,14 +1220,12 @@ class Camera(AlpacaBase):
         """
         List of available readout modes.
         """
-
         return self.get("readoutmodes")
 
     def sensorname(self):
         """
         Name of the sensor used within the camera.
         """
-
         return self.get("sensorname")
 
     def sensortype(self):
@@ -1272,7 +1241,6 @@ class Camera(AlpacaBase):
             Value indicating whether the sensor is monochrome, or what Bayer matrix it
             encodes.
         """
-
         return self.get("sensortype")
 
     def setccdtemperature(self, SetCCDTemperature=None):
@@ -1284,7 +1252,6 @@ class Camera(AlpacaBase):
         :return:
             Camera's cooler setpoint (degrees Celsius).
         """
-
         if SetCCDTemperature is None:
             return self.get("setccdtemperature")
 
@@ -1300,7 +1267,6 @@ class Camera(AlpacaBase):
             Sets the subframe start position for the X axis (0 based) and returns the
             current value. If binning is active, value is in binned pixels.
         """
-
         if StartX is None:
             return self.get("startx")
 
@@ -1316,7 +1282,6 @@ class Camera(AlpacaBase):
             Sets the subframe start position for the Y axis (0 based) and returns the
             current value. If binning is active, value is in binned pixels.
         """
-
         if StartY is None:
             return self.get("starty")
 
@@ -1326,7 +1291,6 @@ class Camera(AlpacaBase):
         """
         Abort the current exposure, if any, and returns the camera to Idle state.
         """
-
         self.put("abortexposure")
 
     def pulseguide(self, Direction: int, Duration: int):
@@ -1338,7 +1302,6 @@ class Camera(AlpacaBase):
                 3 = West).
             Duration (int): Duration of movement in milli-seconds.
         """
-
         self.put("pulseguide", Direction=Direction, Duration=Duration)
 
     def startexposure(self, Duration=0, Light=True):
@@ -1352,7 +1315,6 @@ class Camera(AlpacaBase):
             Duration (float): Duration of exposure in seconds.
             Light (bool): True if light frame, false if dark frame.
         """
-
         self.put("startexposure", Duration=Duration, Light=Light)
 
     def stopexposure(self):
@@ -1363,7 +1325,6 @@ class Camera(AlpacaBase):
             If an exposure is in progress, the readout process is initiated. Ignored if
             readout is already in process.
         """
-
         self.put("stopexposure")
 
 
@@ -1417,7 +1378,6 @@ class Telescope(AlpacaBase):
             Alignment mode of the mount (Alt/Az, Polar, German Polar).
 
         """
-
         return self.get("alignmentmode")
 
     def altitude(self):
@@ -1426,7 +1386,6 @@ class Telescope(AlpacaBase):
         :return:
             Altitude of the telescope's current position (degrees, positive up).
         """
-
         return self.get("altitude")
 
     def aperturearea(self):
@@ -1435,7 +1394,6 @@ class Telescope(AlpacaBase):
         :return:
             Area of the telescope's aperture (square meters).
         """
-
         return self.get("aperturearea")
 
     def aperturediameter(self):
@@ -1444,7 +1402,6 @@ class Telescope(AlpacaBase):
         :return:
             Telescope's effective aperture diameter (meters).
         """
-
         value = self.get("aperturediameter")
         if value:
             return value * 1000
