@@ -17,9 +17,9 @@
 # standard libraries
 import json
 import os
-from threading import Lock
 
 # external packages
+from PyQt5.QtWidgets import QApplication
 from matplotlib import pyplot as plt
 from matplotlib import ticker
 from matplotlib.colors import Normalize
@@ -29,7 +29,6 @@ from scipy.stats.mstats import winsorize
 # local import
 from gui.utilities import toolsQtWidget
 from gui.widgets import analyse_ui
-from base.tpool import Worker
 
 
 class AnalyseWindow(toolsQtWidget.MWidget):
@@ -47,11 +46,7 @@ class AnalyseWindow(toolsQtWidget.MWidget):
 
         self.ui = analyse_ui.Ui_AnalyseDialog()
         self.ui.setupUi(self)
-        self.initUI()
-        self.threadPool = app.threadPool
-        self.threadCounter = 0
-        self.threadingLock = Lock()
-        self.workers = []
+        self.closing = False
 
         self.latitude = None
         self.pierside = None
@@ -111,18 +106,17 @@ class AnalyseWindow(toolsQtWidget.MWidget):
         if 'analyseW' not in self.app.config:
             self.app.config['analyseW'] = {}
         config = self.app.config['analyseW']
-        x = config.get('winPosX', 40)
-        y = config.get('winPosY', 40)
-
-        if x > self.screenSizeX:
-            x = 0
-        if y > self.screenSizeY:
-            y = 0
-
-        self.move(x, y)
-        height = config.get('height', 600)
-        width = config.get('width', 800)
+        height = config.get('height', 800)
+        width = config.get('width', 1200)
         self.resize(width, height)
+        x = config.get('winPosX', 0)
+        y = config.get('winPosY', 0)
+        if x > self.screenSizeX - width:
+            x = 0
+        if y > self.screenSizeY - height:
+            y = 0
+        if x != 0 and y != 0:
+            self.move(x, y)
         self.ui.winsorizedLimit.setChecked(config.get('winsorizedLimit', False))
         return True
 
@@ -153,6 +147,7 @@ class AnalyseWindow(toolsQtWidget.MWidget):
         :param closeEvent:
         :return:
         """
+        self.closing = True
         self.storeConfig()
         self.app.showAnalyse.disconnect(self.showAnalyse)
         super().closeEvent(closeEvent)
@@ -186,7 +181,6 @@ class AnalyseWindow(toolsQtWidget.MWidget):
         :param loadFilePath:
         :return: True for test purpose
         """
-
         d = data[0]
         de = data[-1]
 
@@ -622,21 +616,13 @@ class AnalyseWindow(toolsQtWidget.MWidget):
         axe.figure.canvas.draw()
         return True
 
-    def workerDrawAll(self):
+    def drawAll(self):
         """
         :return:
         """
         for chart in self.charts:
+            if self.closing:
+                break
             chart()
-        return True
-
-    def drawAll(self):
-        """
-        :return: true for test purpose
-        """
-        if self.countSequence is None:
-            return False
-
-        worker = Worker(self.workerDrawAll)
-        self.threadPool.start(worker)
+            QApplication.processEvents()
         return True
