@@ -17,11 +17,14 @@
 # standard libraries
 
 # external packages
+import os.path
+
 import PyQt5
 from PyQt5.QtCore import QRect
 import numpy as np
 import matplotlib.patches as mpatches
 import matplotlib.pyplot as plt
+from PIL import Image
 
 # local import
 from gui.utilities import toolsQtWidget
@@ -31,10 +34,10 @@ from gui.extWindows.hemisphereWext import HemisphereWindowExt
 
 class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
     """
-    the hemisphere window class handles all interaction with model points show / edit etc.
-    the z orders is aligned as follows:
-    there are two planes static / moving where the moving is behind the static one. the
-    static on has to be transparent.
+    the hemisphere window class handles all interaction with model points
+    show / edit etc. the z orders is aligned as follows:
+    there are two planes static / moving where the moving is behind the static
+    one. the static on has to be transparent.
 
     on the static plane we have (and set to the z order)
         - horizon               0
@@ -61,21 +64,20 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         self.app = app
         self.ui = hemisphere_ui.Ui_HemisphereDialog()
         self.ui.setupUi(self)
-        self.initUI()
         self.mutexDraw = PyQt5.QtCore.QMutex()
         self.operationMode = 'normal'
         self.statusDAT = None
 
         self.MODE = dict(
             normal=dict(horMarker='None',
-                        horColor=self.M_GREEN,
+                        horColor=self.M_BLUE,
                         buildPColor=self.M_GREEN_H,
                         starSize=6,
                         starColor=self.M_YELLOW_L,
                         starAnnColor=self.M_WHITE_L),
 
             build=dict(horMarker='None',
-                       horColor=self.M_GREEN,
+                       horColor=self.M_BLUE,
                        buildPColor=self.M_PINK_H,
                        starSize=6,
                        starColor=self.M_YELLOW_L,
@@ -89,14 +91,13 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                          starAnnColor=self.M_WHITE_L),
 
             star=dict(horMarker='None',
-                      horColor=self.M_GREEN_LL,
+                      horColor=self.M_BLUE1,
                       buildPColor=self.M_GREEN_L,
                       starSize=12,
                       starColor=self.M_YELLOW_H,
                       starAnnColor=self.M_WHITE_H)
         )
 
-        # attributes to be stored in class
         self.pointerAltAz = None
         self.pointerDome = None
         self.pointsBuild = None
@@ -116,7 +117,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         self.pointsBuildAnnotate = None
         self.pointsPolarBuild = None
         self.pointsPolarBuildAnnotate = None
+        self.imageTerrain = None
         self.closingWindow = False
+        self.hemMouse = None
 
         self.ui.hemisphereMove.stackUnder(self.ui.hemisphere)
         self.hemisphereMat = self.embedMatplot(self.ui.hemisphere)
@@ -128,45 +131,51 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
     def initConfig(self):
         """
-        initConfig read the key out of the configuration dict and stores it to the gui
-        elements. if some initialisations have to be proceeded with the loaded persistent
-        data, they will be launched as well in this method.
-
         :return: True for test purpose
         """
-
         if 'hemisphereW' not in self.app.config:
             self.app.config['hemisphereW'] = {}
-
         config = self.app.config['hemisphereW']
-        x = config.get('winPosX', 60)
-        y = config.get('winPosY', 60)
-
-        if x > self.screenSizeX:
-            x = 0
-
-        if y > self.screenSizeY:
-            y = 0
-
-        self.move(x, y)
         height = config.get('height', 600)
         width = config.get('width', 800)
         self.resize(width, height)
+        x = config.get('winPosX', 0)
+        y = config.get('winPosY', 0)
+        if x > self.screenSizeX - width:
+            x = 0
+        if y > self.screenSizeY - height:
+            y = 0
+        if x != 0 and y != 0:
+            self.move(x, y)
         self.ui.checkShowSlewPath.setChecked(config.get('checkShowSlewPath', False))
         self.ui.checkShowMeridian.setChecked(config.get('checkShowMeridian', False))
         self.ui.checkShowCelestial.setChecked(config.get('checkShowCelestial', False))
         self.ui.checkShowAlignStar.setChecked(config.get('checkShowAlignStar', False))
         self.ui.checkUseHorizon.setChecked(config.get('checkUseHorizon', True))
         self.ui.showPolar.setChecked(config.get('showPolar', False))
+        self.ui.checkUseTerrain.setChecked(config.get('useTerrain', False))
+        self.ui.terrainAlpha.setValue(config.get('terrainAlpha', 0.35))
+        self.ui.azimuthShift.setValue(config.get('azimuthShift', 0))
+
+        terrainFile = self.app.mwGlob['configDir'] + '/terrain.jpg'
+        if not os.path.isfile(terrainFile):
+            self.imageTerrain = None
+            return False
+
+        img = Image.open(terrainFile).convert('LA')
+        (w, h) = img.size
+        img = img.crop((0, 0, w, h / 2))
+        img = img.resize((1440, 360))
+        img = img.transpose(Image.FLIP_TOP_BOTTOM)
+
+        self.imageTerrain = Image.new('L', (2880, 360))
+        self.imageTerrain.paste(img)
+        self.imageTerrain.paste(img, (1440, 0))
 
         return True
 
     def storeConfig(self):
         """
-        storeConfig writes the keys to the configuration dict and stores. if some
-        saving has to be proceeded to persistent data, they will be launched as
-        well in this method.
-
         :return: True for test purpose
         """
         if 'hemisphereW' not in self.app.config:
@@ -183,7 +192,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         config['checkShowAlignStar'] = self.ui.checkShowAlignStar.isChecked()
         config['checkUseHorizon'] = self.ui.checkUseHorizon.isChecked()
         config['showPolar'] = self.ui.showPolar.isChecked()
-
+        config['useTerrain'] = self.ui.checkUseTerrain.isChecked()
+        config['terrainAlpha'] = self.ui.terrainAlpha.value()
+        config['azimuthShift'] = self.ui.azimuthShift.value()
         return True
 
     def closeEvent(self, closeEvent):
@@ -191,18 +202,18 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         :param closeEvent:
         :return:
         """
+        self.ui.checkEditNone.setChecked(True)
+        self.setOperationMode()
+        self.storeConfig()
         self.closingWindow = True
         self.app.update10s.disconnect(self.updateAlignStar)
         self.app.update1s.disconnect(self.hemisphereMatMove.figure.canvas.draw)
+        self.app.update1s.disconnect(self.polarMatMove.figure.canvas.draw)
         self.app.redrawHemisphere.disconnect(self.drawHemisphere)
         self.app.updatePointMarker.disconnect(self.updatePointMarker)
         self.app.updatePointMarker.disconnect(self.updatePolarPointMarker)
         self.app.mount.signals.settingDone.disconnect(self.updateOnChangedParams)
-        self.storeConfig()
-
-        self.ui.checkEditNone.setChecked(True)
-        self.setOperationMode()
-
+        self.app.enableEditPoints.disconnect(self.enableEditPoints)
         self.ui.checkShowSlewPath.clicked.disconnect(self.drawHemisphere)
         self.ui.checkShowMeridian.clicked.disconnect(self.drawHemisphere)
         self.ui.checkShowCelestial.clicked.disconnect(self.drawHemisphere)
@@ -212,6 +223,10 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         self.ui.checkEditBuildPoints.clicked.disconnect(self.setOperationMode)
         self.ui.checkPolarAlignment.clicked.disconnect(self.setOperationMode)
         self.ui.checkShowAlignStar.clicked.disconnect(self.drawHemisphere)
+        self.ui.checkUseTerrain.clicked.disconnect(self.drawHemisphere)
+        self.ui.azimuthShift.valueChanged.disconnect(self.drawHemisphere)
+        self.ui.terrainAlpha.valueChanged.disconnect(self.drawHemisphere)
+
         self.ui.showPolar.clicked.disconnect(self.togglePolar)
         self.app.mount.signals.pointDone.disconnect(self.updatePointerAltAz)
         self.app.mount.signals.pointDone.disconnect(self.updatePointerPolarAltAz)
@@ -228,10 +243,6 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
     def resizeEvent(self, event):
         """
-        we are using the resize event to reset the timer, which means waiting for
-        RESIZE_FINISHED_TIMEOUT in total before redrawing the complete hemisphere.
-        as we are using a 0.1s cyclic timer.
-
         :param event:
         :return:
         """
@@ -249,10 +260,8 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
     def showWindow(self):
         """
-
         :return:
         """
-
         self.app.update10s.connect(self.updateAlignStar)
         self.app.update1s.connect(self.hemisphereMatMove.figure.canvas.draw)
         self.app.update1s.connect(self.polarMatMove.figure.canvas.draw)
@@ -265,7 +274,7 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         self.app.dome.signals.azimuth.connect(self.updateDome)
         self.app.dome.signals.deviceDisconnected.connect(self.updateDome)
         self.app.dome.signals.serverDisconnected.connect(self.updateDome)
-
+        self.app.enableEditPoints.connect(self.enableEditPoints)
         self.ui.checkShowSlewPath.clicked.connect(self.drawHemisphere)
         self.ui.checkUseHorizon.clicked.connect(self.drawHemisphere)
         self.ui.checkShowAlignStar.clicked.connect(self.drawHemisphere)
@@ -275,24 +284,23 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         self.ui.checkEditHorizonMask.clicked.connect(self.setOperationMode)
         self.ui.checkEditBuildPoints.clicked.connect(self.setOperationMode)
         self.ui.checkPolarAlignment.clicked.connect(self.setOperationMode)
+        self.ui.checkUseTerrain.clicked.connect(self.drawHemisphere)
+        self.ui.azimuthShift.valueChanged.connect(self.drawHemisphere)
+        self.ui.terrainAlpha.valueChanged.connect(self.drawHemisphere)
+
         self.ui.showPolar.clicked.connect(self.togglePolar)
         self.ui.addPositionToHorizon.clicked.connect(self.addHorizonPointManual)
-
-        self.hemisphereMat.figure.canvas.mpl_connect('button_press_event',
-                                                     self.onMouseDispatcher)
-        self.hemisphereMat.figure.canvas.mpl_connect('motion_notify_event',
-                                                     self.showMouseCoordinates)
+        hem = self.hemisphereMat.figure.canvas
+        self.hemMouse = hem.mpl_connect('button_press_event', self.onMouseDispatcher)
+        hem.mpl_connect('motion_notify_event', self.showMouseCoordinates)
         self.togglePolar()
         self.show()
-
         return True
 
     def togglePolar(self):
         """
-
         :return: success
         """
-
         if self.ui.showPolar.isChecked():
             self.ui.polar.setMaximumSize(16777215, 16777215)
             self.ui.polar.setVisible(True)
@@ -302,18 +310,14 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             self.ui.polar.setVisible(False)
 
         self.drawHemisphere()
-
         return True
 
     def updateOnChangedParams(self, sett):
         """
-
         :param sett:
         :return: status redraw
         """
-
         needRedraw = False
-
         if self.meridianSlewParam != sett.meridianLimitSlew:
             self.meridianSlewParam = sett.meridianLimitSlew
             needRedraw = True
@@ -339,12 +343,10 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         """
         :return: success
         """
-
         if self.pointerAltAz is None:
             return False
 
         obsSite = self.app.mount.obsSite
-
         if obsSite.Alt is None or obsSite.Az is None:
             self.pointerAltAz.set_visible(False)
             return False
@@ -355,19 +357,16 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         alt = obsSite.Alt.degrees
         az = obsSite.Az.degrees
         self.pointerAltAz.set_data((az, alt))
-
         return True
 
     def updatePointerPolarAltAz(self):
         """
         :return: success
         """
-
         if self.pointerPolarAltAz is None:
             return False
 
         obsSite = self.app.mount.obsSite
-
         if obsSite.Alt is None or obsSite.Az is None:
             self.pointerPolarAltAz.set_visible(False)
             return False
@@ -378,19 +377,17 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         alt = 90 - obsSite.Alt.degrees
         az = obsSite.Az.radians
         self.pointerPolarAltAz.set_data((az, alt))
-
         return True
 
     def updateDome(self, azimuth):
         """
-        updateDome is called whenever an update of coordinates from dome are given.
-        it takes the actual values and corrects the point in window if window is in
-        show status.
+        updateDome is called whenever an update of coordinates from dome are
+        given. it takes the actual values and corrects the point in window if
+        window is in show status.
 
         :param azimuth:
         :return: success
         """
-
         if self.pointerDome is None:
             return False
 
@@ -399,57 +396,47 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             return False
 
         visible = self.app.deviceStat['dome']
-
         self.pointerDome.set_xy((azimuth - 15, 1))
         self.pointerDome.set_visible(visible)
-
         return True
 
     def getMarkerStatusParams(self, active, index):
         """
-
         :param active:
         :param index:
         :return: values
         """
-
         if active:
             marker = self.markerPoint()
             color = self.MODE[self.operationMode]['buildPColor']
-            markersize = 9
+            mSize = 9
             annotationText = f'{index + 1:2d}'
 
         else:
             marker = '$\u2714$'
             color = self.M_YELLOW
-            markersize = 11
+            mSize = 11
             annotationText = f'{index + 1:2d}'
 
-        return marker, markersize, color, annotationText
+        return marker, mSize, color, annotationText
 
     def updatePointMarker(self):
         """
-
         :return: success
         """
-
         for index, point in enumerate(self.app.data.buildP):
             active = point[2]
-
-            marker, markersize, color, _ = self.getMarkerStatusParams(active, index)
-
+            marker, mSize, color, _ = self.getMarkerStatusParams(active, index)
             self.pointsBuild[index].set_marker(marker)
-            self.pointsBuild[index].set_markersize(markersize)
+            self.pointsBuild[index].set_markersize(mSize)
             self.pointsBuild[index].set_color(color)
             self.pointsBuildAnnotate[index].set_color(color)
 
         self.hemisphereMat.figure.canvas.draw()
-
         return True
 
     def updatePolarPointMarker(self):
         """
-
         :return: success
         """
         if not self.pointsPolarBuild:
@@ -457,37 +444,32 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
         for index, point in enumerate(self.app.data.buildP):
             active = point[2]
-
-            marker, markersize, color, _ = self.getMarkerStatusParams(active, index)
-
+            marker, mSize, color, _ = self.getMarkerStatusParams(active, index)
             self.pointsPolarBuild[index].set_marker(marker)
-            self.pointsPolarBuild[index].set_markersize(markersize)
+            self.pointsPolarBuild[index].set_markersize(mSize)
             self.pointsPolarBuild[index].set_color(color)
             self.pointsPolarBuildAnnotate[index].set_color(color)
 
         self.polarMat.figure.canvas.draw()
-
         return True
 
     def updateAlignStar(self):
         """
-        updateAlignStar is called whenever an update of coordinates from mount are
-        given. it takes the actual values and corrects the point in window if window is in
-        show status.
-        If the object is not created, the routing returns false.
-
-        due to the fact that all annotation are only shown if in axes when coordinate
-        are in data, after some time, no annotation will be shown, because just moved.
-        therefore we add each time the annotation again.
+        updateAlignStar is called whenever an update of coordinates from mount
+        are given. it takes the actual values and corrects the point in window if
+        window is in show status. If the object is not created, the routing
+        returns false. Due to the fact that all annotation are only shown if in
+        axes when coordinate are in data, after some time, no annotation will be
+        shown, because just moved. Therefore we add each time the annotation again.
 
         :return: success
         """
-
         if self.starsAlign is None:
             return False
-
         if self.starsAlignAnnotate is None:
             return False
+        if not self.mutexDraw.tryLock():
+            return False
 
         axes = self.hemisphereMat.figure.axes[0]
         hip = self.app.hipparcos
@@ -512,26 +494,23 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             self.starsAlignAnnotate.append(annotation)
 
         self.hemisphereMat.figure.canvas.draw()
-
+        self.mutexDraw.unlock()
         return True
 
     def clearHemisphere(self):
         """
-        clearHemisphere is called when after startup the location of the mount is changed
-        to reconstruct correctly the hemisphere window
+        clearHemisphere is called when after startup the location of the mount
+        is changed to reconstruct correctly the hemisphere window
 
         :return: True for test Purpose
         """
-
         self.pointsBuild = None
         self.app.data.clearBuildP()
         self.drawHemisphere()
-
         return True
 
     def staticHorizon(self, axes=None, polar=False):
         """
-
         :param axes:
         :param polar:
         :return:
@@ -555,19 +534,19 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
             axes.plot(az,
                       90 - alt,
-                      color=self.MODE['normal']['horColor'],
-                      marker=self.MODE[self.operationMode]['horMarker'],
+                      color=self.MODE[self.operationMode]['horColor'],
+                      marker=self.MODE['normal']['horMarker'],
                       alpha=0.5,
                       zorder=0,
                       ls='none')
 
             axes.plot(azF,
                       90 - altF,
-                      color=self.MODE['normal']['horColor'],
+                      color=self.MODE[self.operationMode]['horColor'],
                       marker='',
-                      alpha=0.5,
+                      alpha=0.7,
                       zorder=0,
-                      lw=3)
+                      lw=2)
 
         else:
             alt = np.array(alt)
@@ -576,7 +555,7 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             azF = np.concatenate([[0], [0], az, [360], [360]])
             axes.fill(azF,
                       altF,
-                      color=self.M_GREEN_LL,
+                      color=self.M_GREY_LL,
                       alpha=0.5,
                       zorder=0)
 
@@ -592,15 +571,14 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                       alt,
                       color=self.MODE[self.operationMode]['horColor'],
                       marker='',
-                      alpha=0.5,
+                      alpha=0.7,
                       zorder=0,
-                      lw=3)
+                      lw=2)
 
         return True
 
     def staticModelData(self, axes=None, polar=False):
         """
-
         :param axes:
         :param polar:
         :return:
@@ -623,11 +601,11 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             alt = point[0]
             active = point[2]
 
-            marker, markersize, color, text = self.getMarkerStatusParams(active, index)
+            marker, mSize, color, text = self.getMarkerStatusParams(active, index)
 
             if self.ui.checkShowSlewPath.isChecked() and index > 0:
                 if polar:
-                    axes.plot(np.radians((points[index - 1][1], points[index][1])),
+                    axes.plot(np.radians((-points[index - 1][1], -points[index][1])),
                               (90 - points[index - 1][0], 90 - points[index][0]),
                               ls=':', lw=1, color=self.M_WHITE, zorder=40)
 
@@ -637,9 +615,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                               ls=':', lw=1, color=self.M_WHITE, zorder=40)
 
             if polar:
-                p, = axes.plot(np.radians(az), 90 - alt,
+                p, = axes.plot(np.radians(-az), 90 - alt,
                                marker=marker,
-                               markersize=markersize,
+                               markersize=mSize,
                                fillstyle='none',
                                color=self.MODE['normal']['buildPColor'],
                                zorder=40,
@@ -649,7 +627,7 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
             else:
                 p, = axes.plot(az, alt,
                                marker=marker,
-                               markersize=markersize,
+                               markersize=mSize,
                                fillstyle='none',
                                color=self.MODE[self.operationMode]['buildPColor'],
                                zorder=40,
@@ -658,7 +636,7 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
             if polar:
                 annotation = axes.annotate(text,
-                                           xy=(np.radians(az), 90 - alt),
+                                           xy=(np.radians(-az), 90 - alt),
                                            xytext=(2, -10),
                                            textcoords='offset points',
                                            color=color,
@@ -690,9 +668,10 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
         alt, az = zip(*celestial)
         alt = np.array(alt)
+        az = np.array(az)
 
         if polar:
-            self.celestialPath, = axes.plot(np.radians(az),
+            self.celestialPath, = axes.plot(np.radians(-az),
                                             90 - alt,
                                             '.',
                                             markersize=1,
@@ -711,11 +690,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
     def staticMeridianLimits(self, axes=None):
         """
-
         :param axes:
         :return:
         """
-
         slew = self.app.mount.setting.meridianLimitSlew
         track = self.app.mount.setting.meridianLimitTrack
 
@@ -737,12 +714,10 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                                                 color=self.M_YELLOW_L,
                                                 alpha=0.5)
         axes.add_patch(self.meridianTrack)
-
         return True
 
     def staticHorizonLimits(self, axes=None, polar=False):
         """
-
         :param axes:
         :param polar:
         :return:
@@ -776,13 +751,41 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                                                   visible=True)
         axes.add_patch(self.horizonLimitHigh)
         axes.add_patch(self.horizonLimitLow)
+        return True
 
+    def staticTerrainMask(self, axes=None, polar=False):
+        """
+        :param axes:
+        :param polar:
+        :return:
+        """
+        if not self.imageTerrain:
+            return False
+
+        shift = self.ui.azimuthShift.value()
+        alpha = self.ui.terrainAlpha.value()
+        imgF = self.imageTerrain.crop((4 * shift, 0, 1440 + 4 * shift, 360))
+        (w, h) = imgF.size
+        img = list(imgF.getdata())
+        img = np.array(img).reshape((h, w))
+
+        if polar:
+            phi = np.linspace(0, 2 * np.pi, img.shape[1] + 1)
+            r = np.linspace(0, 90, img.shape[0] + 1)
+
+            Phi, R = np.meshgrid(phi, r)
+            axes.pcolormesh(Phi, 90 - R, img[:, :], linewidth=0, linestyle='None',
+                            zorder=-10, cmap='gray', alpha=alpha * 0.1)
+
+        else:
+            axes.imshow(img, aspect='auto', extent=(0, 360, 90, 0),
+                        zorder=-10, cmap='gray', alpha=alpha)
         return True
 
     def drawHemisphereStatic(self, axes=None, polar=False):
         """
-         drawHemisphereStatic renders the static part of the hemisphere window and puts
-         all drawing on the static plane. the content consist of:
+         drawHemisphereStatic renders the static part of the hemisphere window
+         and puts all drawing on the static plane. the content consist of:
             - modeldata points
             - horizon mask
             - celestial paths
@@ -802,6 +805,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
             self.staticModelData(axes=axes, polar=polar)
 
+            if self.ui.checkUseTerrain.isChecked():
+                self.staticTerrainMask(axes=axes, polar=polar)
+
         else:
             if self.ui.checkUseHorizon.isChecked():
                 self.staticHorizon(axes=axes)
@@ -815,6 +821,9 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
 
             self.staticModelData(axes=axes, polar=polar)
 
+            if self.ui.checkUseTerrain.isChecked():
+                self.staticTerrainMask(axes=axes)
+
         return True
 
     def drawHemisphereMoving(self, axes=None, polar=False):
@@ -822,14 +831,13 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         drawHemisphereMoving is rendering the moving part which consists of:
             - pointer: where the mount points to
             - dome widget: which shows the position of the dome opening
-        the dynamic ones are located on a separate plane to improve rendering speed,
-        because we update this part very often.
+        the dynamic ones are located on a separate plane to improve rendering
+        speed, because we update this part very often.
 
         :param axes: matplotlib axes object
         :param polar: flag if polar should be drawn
         :return:
         """
-
         if polar:
             self.pointerPolarAltAz, = axes.plot(np.radians(180), 45,
                                                 zorder=10,
@@ -864,18 +872,19 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                                                   fill=True,
                                                   visible=False)
             axes.add_patch(self.pointerDome)
-
         return True
 
     def drawAlignmentStars(self, axes=None):
         """
-        drawAlignmentStars is rendering the alignment star map. this moves over time with
-        the speed of earth turning. so we have to update the rendering, but on low speed
-        without having any user interaction.
+        drawAlignmentStars is rendering the alignment star map. this moves over
+        time with the speed of earth turning. so we have to update the rendering,
+        but on low speed without having any user interaction.
 
         :param axes: matplotlib axes object
         :return: true for test purpose
         """
+        if not self.mutexDraw.tryLock():
+            return False
 
         self.starsAlignAnnotate = list()
         hip = self.app.hipparcos
@@ -902,19 +911,20 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
                                        clip_on=True,
                                        )
             self.starsAlignAnnotate.append(annotation)
-
+        self.mutexDraw.unlock()
         return True
 
     def drawHemisphere(self):
         """
-        drawHemisphere is the basic renderer for all items and widgets in the hemisphere
-        window. it takes care of drawing the grid, enables three layers of transparent
-        widgets for static content, moving content and star maps. this is mainly done to
-        get a reasonable performance when redrawing the canvas. in addition it initializes
-        the objects for points markers, patches, lines etc. for making the window nice
-        and user friendly.
-        the user interaction on the hemisphere windows is done by the event handler of
-        matplotlib itself implementing an on Mouse handler, which takes care of functions.
+        drawHemisphere is the basic renderer for all items and widgets in the
+        hemisphere window. it takes care of drawing the grid, enables three
+        layers of transparent widgets for static content, moving content and star
+        maps. this is mainly done to get a reasonable performance when redrawing
+        the canvas. in addition it initializes the objects for points markers,
+        patches, lines etc. for making the window nice and user friendly. the
+        user interaction on the hemisphere windows is done by the event handler
+        of matplotlib itself implementing an on Mouse handler, which takes care
+        of functions.
 
         :return: True for test purpose
         """
@@ -929,10 +939,12 @@ class HemisphereWindow(toolsQtWidget.MWidget, HemisphereWindowExt):
         hasPolar = self.ui.showPolar.isChecked()
         if hasPolar:
             axePolar, _ = self.generatePolar(widget=self.polarMat,
-                                             horizon=True)
+                                             horizon=True,
+                                             reverse=True)
             axePolarMove, _ = self.generatePolar(widget=self.polarMatMove,
                                                  horizon=True,
-                                                 showAxes=False)
+                                                 showAxes=False,
+                                                 reverse=True)
         else:
             self.pointerPolarAltAz = None
             self.pointsBuildAnnotate = None
