@@ -20,6 +20,10 @@ import os
 
 # external packages
 import PyQt5.QtWidgets
+from PyQt5.QtCore import QMutex
+import numpy as np
+from scipy.interpolate import griddata
+from scipy.ndimage import uniform_filter
 from astropy.io import fits
 from astropy import wcs
 from astropy.nddata import Cutout2D
@@ -28,13 +32,12 @@ from astropy.visualization import AsinhStretch
 from astropy.visualization import imshow_norm
 from matplotlib.patches import Ellipse
 import matplotlib.pyplot as plt
-from skyfield.api import Angle
-import numpy as np
 from colour_demosaicing import demosaicing_CFA_Bayer_bilinear
-from mountcontrol.convert import convertToDMS, convertToHMS
 import sep
 
 # local import
+from mountcontrol.convert import convertToDMS, convertToHMS
+from base.fitsHeader import getCoordinates, getSQM, getExposure, getScale
 from gui.utilities import toolsQtWidget
 from gui.widgets import image_ui
 from base.tpool import Worker
@@ -60,11 +63,11 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.threadPool = app.threadPool
         self.ui = image_ui.Ui_ImageDialog()
         self.ui.setupUi(self)
-        self.initUI()
         self.signals = ImageWindowSignals()
 
         self.imageFileName = ''
         self.imageFileNameOld = ''
+        self.preparePlotLock = QMutex()
         self.expTime = 1
         self.binning = 1
         self.image = None
@@ -92,6 +95,7 @@ class ImageWindow(toolsQtWidget.MWidget):
                      4: 'Photometry: Background level',
                      5: 'Photometry: Background noise',
                      6: 'Photometry: Flux',
+                     7: 'Photometry: HFD Contour',
                      }
 
         self.colorMaps = {'Grey': 'gray',
@@ -132,16 +136,17 @@ class ImageWindow(toolsQtWidget.MWidget):
         if 'imageW' not in self.app.config:
             self.app.config['imageW'] = {}
         config = self.app.config['imageW']
-        x = config.get('winPosX', 80)
-        y = config.get('winPosY', 80)
-        if x > self.screenSizeX:
-            x = 0
-        if y > self.screenSizeY:
-            y = 0
-        self.move(x, y)
         height = config.get('height', 600)
         width = config.get('width', 800)
         self.resize(width, height)
+        x = config.get('winPosX', 0)
+        y = config.get('winPosY', 0)
+        if x > self.screenSizeX - width:
+            x = 0
+        if y > self.screenSizeY - height:
+            y = 0
+        if x != 0 and y != 0:
+            self.move(x, y)
 
         self.setupDropDownGui()
         self.ui.color.setCurrentIndex(config.get('color', 0))
@@ -189,9 +194,6 @@ class ImageWindow(toolsQtWidget.MWidget):
 
         :return: true for test purpose
         """
-        self.show()
-        self.showCurrent()
-
         self.ui.load.clicked.connect(self.selectImage)
         self.ui.color.currentIndexChanged.connect(self.preparePlot)
         self.ui.stretch.currentIndexChanged.connect(self.preparePlot)
@@ -203,11 +205,14 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.ui.solve.clicked.connect(self.solveCurrent)
         self.ui.expose.clicked.connect(self.exposeImage)
         self.ui.exposeN.clicked.connect(self.exposeImageN)
+        self.ui.checkStackImages.clicked.connect(self.clearStack)
         self.ui.abortImage.clicked.connect(self.abortImage)
         self.ui.abortSolve.clicked.connect(self.abortSolve)
         self.signals.showCurrent.connect(self.showCurrent)
         self.signals.solveImage.connect(self.solveImage)
         self.app.showImage.connect(self.showImage)
+        self.show()
+        self.showCurrent()
         return True
 
     def closeEvent(self, closeEvent):
@@ -221,7 +226,6 @@ class ImageWindow(toolsQtWidget.MWidget):
         :return: True for test purpose
         """
         self.storeConfig()
-
         self.ui.load.clicked.disconnect(self.selectImage)
         self.ui.color.currentIndexChanged.disconnect(self.preparePlot)
         self.ui.stretch.currentIndexChanged.disconnect(self.preparePlot)
@@ -233,13 +237,12 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.ui.solve.clicked.disconnect(self.solveCurrent)
         self.ui.expose.clicked.disconnect(self.exposeImage)
         self.ui.exposeN.clicked.disconnect(self.exposeImageN)
+        self.ui.checkStackImages.clicked.disconnect(self.clearStack)
         self.ui.abortImage.clicked.disconnect(self.abortImage)
         self.ui.abortSolve.clicked.disconnect(self.abortSolve)
         self.signals.showCurrent.disconnect(self.showCurrent)
         self.signals.solveImage.disconnect(self.solveImage)
-
         self.app.showImage.disconnect(self.showImage)
-
         plt.close(self.imageMat.figure)
         super().closeEvent(closeEvent)
 
@@ -566,8 +569,24 @@ class ImageWindow(toolsQtWidget.MWidget):
             yTicks = plt.getp(colorbar.ax.axes, 'yticklabels')
             plt.setp(yTicks, color=self.M_BLUE, fontweight='bold')
 
-        self.axe.figure.canvas.draw()
+        if self.ui.view.currentIndex() == 7 and self.radius is not None and \
+                self.objs is not None:
 
+            x = self.objs['x']
+            y = self.objs['y']
+            z = self.radius
+            width = imageDisp.shape[1]
+            height = imageDisp.shape[0]
+            X, Y = np.meshgrid(range(0, width, int(width / 250)),
+                               range(0, height, int(height / 250)))
+            Z = griddata((x, y), z, (X, Y), method='linear', fill_value=np.mean(z))
+            Z = uniform_filter(Z, size=25)
+            self.axe.contourf(X, Y, Z, 20)
+
+            if self.axeCB:
+                self.axeCB.axis('off')
+
+        self.axe.figure.canvas.draw()
         return True
 
     def writeHeaderDataToGUI(self):
@@ -577,53 +596,49 @@ class ImageWindow(toolsQtWidget.MWidget):
         name = self.header.get('OBJECT', '').upper()
         self.ui.object.setText(f'{name}')
 
-        ra = Angle(degrees=self.header.get('RA', 0))
-        dec = Angle(degrees=self.header.get('DEC', 0))
+        ra, dec = getCoordinates(header=self.header)
 
-        # ra will be in hours
-        self.ui.ra.setText(f'{ra.hstr(warn=False)}')
-        self.ui.dec.setText(f'{dec.dstr()}')
+        self.ui.ra.setText(f'{self.formatHstrToText(ra)}')
+        self.ui.dec.setText(f'{self.formatDstrToText(dec)}')
 
-        scale = self.header.get('SCALE', 0)
+        scale = getScale(header=self.header)
         rotation = self.header.get('ANGLE', 0)
         self.ui.scale.setText(f'{scale:5.3f}')
         self.ui.rotation.setText(f'{rotation:6.3f}')
 
-        ccdTemp = self.header.get('CCD-TEMP', 0)
-        self.ui.ccdTemp.setText(f'{ccdTemp:4.1f}')
+        value = self.header.get('CCD-TEMP', 0)
+        self.guiSetText(self.ui.ccdTemp, '4.1f', value)
 
-        expTime1 = self.header.get('EXPOSURE', 0)
-        expTime2 = self.header.get('EXPTIME', 0)
-        expTime = max(expTime1, expTime2)
-        self.ui.expTime.setText(f'{expTime:5.1f}')
+        value = getExposure(header=self.header)
+        self.guiSetText(self.ui.expTime, '5.1f', value)
 
-        filterCCD = self.header.get('FILTER', 0)
-        self.ui.filter.setText(f'{filterCCD}')
+        value = self.header.get('FILTER', 0)
+        value = f'{value}'
+        self.guiSetText(self.ui.filter, 's', value)
 
-        binX = self.header.get('XBINNING', 0)
-        binY = self.header.get('YBINNING', 0)
-        self.ui.binX.setText(f'{binX:1.0f}')
-        self.ui.binY.setText(f'{binY:1.0f}')
+        value = self.header.get('XBINNING', 0)
+        self.guiSetText(self.ui.binX, '1.0f', value)
+        value = self.header.get('YBINNING', 0)
+        self.guiSetText(self.ui.binY, '1.0f', value)
 
-        sqm = max(self.header.get('SQM', 0),
-                  self.header.get('SKY-QLTY', 0),
-                  self.header.get('MPSAS', 0),
-                  )
-        self.ui.sqm.setText(f'{sqm:5.2f}')
+        value = getSQM(header=self.header)
+        self.guiSetText(self.ui.sqm, '5.2f', value)
 
         flipped = bool(self.header.get('FLIPPED', False))
         self.ui.isFlipped.setEnabled(flipped)
 
         return True
 
-    def preparePlot(self):
+    def workerPreparePlot(self):
         """
         :return:
         """
         if self.image is None:
+            self.preparePlotLock.unlock()
             return False
 
         if self.header is None:
+            self.preparePlotLock.unlock()
             return False
 
         self.updateWindowsStats()
@@ -653,6 +668,17 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.stretchImage()
         self.colorImage()
         self.imagePlot()
+        self.preparePlotLock.unlock()
+        return True
+
+    def preparePlot(self):
+        """
+        :return:
+        """
+        if not self.preparePlotLock.tryLock():
+            return False
+        worker = Worker(self.workerPreparePlot)
+        self.threadPool.start(worker)
         return True
 
     def workerPhotometry(self):
@@ -703,8 +729,6 @@ class ImageWindow(toolsQtWidget.MWidget):
         if np.shape(self.image) != np.shape(self.imageStack):
             self.imageStack = None
 
-        self.ui.numberStacks.setText(f'mean of: {self.numberStack:4.0f}')
-
         if self.imageStack is None:
             self.imageStack = self.image
             self.numberStack = 1
@@ -714,6 +738,19 @@ class ImageWindow(toolsQtWidget.MWidget):
             self.numberStack += 1
 
         self.image = self.imageStack / self.numberStack
+        self.ui.numberStacks.setText(f'mean of: {self.numberStack:4.0f}')
+        return True
+
+    def clearStack(self):
+        """
+        :return:
+        """
+        if not self.ui.checkStackImages.isChecked():
+            self.imageStack = None
+            self.numberStack = 0
+            self.ui.numberStacks.setText('single')
+            return False
+
         return True
 
     def zoomImage(self):
@@ -753,6 +790,7 @@ class ImageWindow(toolsQtWidget.MWidget):
         :param imagePath:
         :return:
         """
+        import time
         if not imagePath:
             return False
 
@@ -859,6 +897,7 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.deviceStat['expose'] = True
         self.ui.checkStackImages.setChecked(False)
         self.app.camera.signals.saved.connect(self.exposeImageDone)
+        self.ui.numberStacks.setText('...')
         self.exposeRaw()
         return True
 
@@ -867,7 +906,7 @@ class ImageWindow(toolsQtWidget.MWidget):
         :param imagePath:
         :return: True for test purpose
         """
-        text = f'Exposed:            [{os.path.basename(imagePath)}]'
+        text = f'Exposed:             [{os.path.basename(imagePath)}]'
         self.app.message.emit(text, 0)
 
         if self.ui.checkAutoSolve.isChecked():
@@ -888,6 +927,7 @@ class ImageWindow(toolsQtWidget.MWidget):
         self.imageStack = None
         self.deviceStat['exposeN'] = True
         self.app.camera.signals.saved.connect(self.exposeImageNDone)
+        self.ui.numberStacks.setText('...')
         self.exposeRaw()
         return True
 
@@ -955,16 +995,15 @@ class ImageWindow(toolsQtWidget.MWidget):
         """
         if not imagePath:
             return False
-
         if not os.path.isfile(imagePath):
             return False
 
         updateFits = self.ui.checkEmbedData.isChecked()
+        self.app.astrometry.signals.done.connect(self.solveDone)
         self.app.astrometry.solveThreading(fitsPath=imagePath,
                                            updateFits=updateFits,
                                            )
         self.deviceStat['solve'] = True
-        self.app.astrometry.signals.done.connect(self.solveDone)
         text = f'Solving:             [{os.path.basename(imagePath)}]'
         self.app.message.emit(text, 0)
         return True
