@@ -26,6 +26,7 @@ import numpy as np
 import matplotlib.pyplot
 
 # local import
+from logic.modeldata.modelHandling import writeRetrofitData
 
 
 class ManageModel(object):
@@ -119,7 +120,15 @@ class ManageModel(object):
         pointsOut = []
         for buildPoint in buildModel:
             for mountPoint in mountModel:
-                if mountModel[mountPoint] == buildModel[buildPoint]:
+                dHA = mountModel[mountPoint]['ha'] - buildModel[buildPoint]['ha']
+                dHA = dHA / mountModel[mountPoint]['ha']
+                dDEC = mountModel[mountPoint]['dec'] - buildModel[buildPoint]['dec']
+                dDEC = dDEC / mountModel[mountPoint]['dec']
+
+                fitHA = abs(dHA) < 1e-4
+                fitDEC = abs(dDEC) < 1e-4
+
+                if fitHA and fitDEC:
                     pointsIn.append(buildPoint)
                     break
 
@@ -173,7 +182,7 @@ class ManageModel(object):
 
             pointsIn, pointsOut = self.compareModel(buildModelData, mountModel)
 
-            if pointsIn:
+            if len(pointsIn) > 2:
                 self.fittedModelPoints = pointsIn
                 self.fittedModelPath = modelFilePath
                 break
@@ -496,7 +505,7 @@ class ManageModel(object):
                 continue
             newModel.append(element)
 
-        newModel = self.writeRetrofitData(self.app.mount.model, newModel)
+        newModel = writeRetrofitData(self.app.mount.model, newModel)
 
         with open(newPath, 'w+') as newFile:
             json.dump(newModel, newFile, sort_keys=True, indent=4)
