@@ -30,6 +30,7 @@ from mountcontrol.convert import convertToHMS, convertToDMS
 # local import
 from base import transform
 from gui.utilities.toolsQtWidget import QMultiWait
+from logic.modeldata.modelHandling import writeRetrofitData
 
 
 class Model:
@@ -64,13 +65,14 @@ class Model:
         self.ui.endModel.clicked.connect(self.processModelData)
         self.ui.pauseModel.clicked.connect(self.pauseBuild)
         self.ui.batchModel.clicked.connect(self.loadProgramModel)
+        self.ui.plateSolveSync.clicked.connect(self.plateSolveSync)
 
     def initConfig(self):
         """
         :return: True for test purpose
         """
         config = self.app.config['mainW']
-        self.ui.checkDisableDAT.setChecked(config.get('checkDisableDAT', True))
+        self.ui.retriesReverse.setChecked(config.get('retriesReverse', False))
         self.ui.parkMountAfterModel.setChecked(config.get('parkMountAfterModel', False))
         self.ui.numberBuildRetries.setValue(config.get('numberBuildRetries', 0))
         return True
@@ -80,7 +82,7 @@ class Model:
         :return: True for test purpose
         """
         config = self.app.config['mainW']
-        config['checkDisableDAT'] = self.ui.checkDisableDAT.isChecked()
+        config['retriesReverse'] = self.ui.retriesReverse.isChecked()
         config['parkMountAfterModel'] = self.ui.parkMountAfterModel.isChecked()
         config['numberBuildRetries'] = self.ui.numberBuildRetries.value()
         return True
@@ -114,37 +116,29 @@ class Model:
 
     def updateTurnKnobsGUI(self, model):
         """
-        updateTurnKnobsGUI shows the data which is received through the getain
-        command. this is mainly polar and ortho errors as well as basic model
-        data.
-
         :param model:
         :return:    True if ok for testing
         """
         if model.azimuthTurns is not None:
             if model.azimuthTurns > 0:
-                text = '{0:3.2f} revs left'.format(abs(model.azimuthTurns))
-
+                text = '{0:3.1f} revs left'.format(abs(model.azimuthTurns))
             else:
-                text = '{0:3.2f} revs right'.format(abs(model.azimuthTurns))
+                text = '{0:3.1f} revs right'.format(abs(model.azimuthTurns))
 
         else:
             text = '-'
 
         self.ui.azimuthTurns.setText(text)
-
         if model.altitudeTurns is not None:
             if model.altitudeTurns > 0:
-                text = '{0:3.2f} revs down'.format(abs(model.altitudeTurns))
-
+                text = '{0:3.1f} revs down'.format(abs(model.altitudeTurns))
             else:
-                text = '{0:3.2f} revs up'.format(abs(model.altitudeTurns))
+                text = '{0:3.1f} revs up'.format(abs(model.altitudeTurns))
 
         else:
             text = '-'
 
         self.ui.altitudeTurns.setText(text)
-
         return True
 
     def updateProgress(self, number=0, count=0):
@@ -191,16 +185,27 @@ class Model:
         :param result: true for test purpose
         :return: success
         """
-        self.log.debug('Processing astrometry result')
-        if self.resultQueue.empty():
-            self.log.info('Empty result queue')
+        noResultsLeft = self.resultQueue.empty()
+        slewsLeft = not self.slewQueue.empty()
+        imagesLeft = not self.imageQueue.empty()
+        solvesLeft = not self.solveQueue.empty()
+        stillToWork = slewsLeft or imagesLeft or solvesLeft
+
+        if noResultsLeft and stillToWork:
+            self.log.error('Empty result queue -> error')
+            t = f'Slews left: [{self.slewQueue.qsize()}] '
+            t += f'Images left: [{self.imageQueue.qsize()}] '
+            t += f'Solves left: [{self.solveQueue.qsize()}] '
+            self.log.error(t)
             return False
 
+        self.log.debug('Processing astrometry result')
         mPoint = self.resultQueue.get()
         self.log.debug(f'Result from queue [{mPoint["countSequence"]:03d}]: [{mPoint}]')
 
-        number = mPoint["lenSequence"]
+        lenSequence = mPoint["lenSequence"]
         count = mPoint["countSequence"]
+        pointNumber = mPoint["pointNumber"]
 
         if not result:
             self.log.debug('Solving result is missing')
@@ -208,16 +213,18 @@ class Model:
 
         mPoint.update(result)
         isSuccess = mPoint['success']
-        isInRange = mPoint['errorRMS_S'] < self.MAX_ERROR_MODEL_POINT
+
+        isInRange = mPoint.get('errorRMS_S', 0) < self.MAX_ERROR_MODEL_POINT
         if isSuccess and isInRange:
             raJNowS, decJNowS = transform.J2000ToJNow(mPoint['raJ2000S'],
                                                       mPoint['decJ2000S'],
                                                       mPoint['julianDate'])
             mPoint['raJNowS'] = raJNowS
             mPoint['decJNowS'] = decJNowS
-            self.log.debug(f'Queued to model [{mPoint["countSequence"]:03d}]: [{mPoint}]')
+            t = f'Queued to model [{mPoint["countSequence"]:03d}]: [{mPoint}]'
+            self.log.debug(t)
             self.modelQueue.put(mPoint)
-            self.app.data.setStatusBuildP(count - 1, False)
+            self.app.data.setStatusBuildP(pointNumber - 1, False)
             self.app.updatePointMarker.emit()
 
             text = f'Solved   image-{count:03d}:  '
@@ -238,8 +245,8 @@ class Model:
             self.app.message.emit(text, 2)
             self.retryQueue.put(mPoint)
 
-        self.updateProgress(number=number, count=count)
-        if number == count:
+        self.updateProgress(number=lenSequence, count=count)
+        if lenSequence == count:
             self.modelCycleThroughBuildPointsFinished()
 
         return True
@@ -258,22 +265,24 @@ class Model:
 
         :return: success
         """
-        self.log.debug('Solving started')
-        if self.solveQueue.empty():
-            self.log.info('Empty solve queue')
+        noSolvesLeft = self.solveQueue.empty()
+        slewsLeft = not self.slewQueue.empty()
+        imagesLeft = not self.imageQueue.empty()
+        stillToWork = slewsLeft or imagesLeft
+
+        if noSolvesLeft and stillToWork:
+            t = f'Slews left: [{self.slewQueue.qsize()}] '
+            t += f'Images left: [{self.imageQueue.qsize()}] '
+            self.log.error(f'Empty solve queue: {t}')
             return False
 
+        self.log.debug('Solving started')
         mPoint = self.solveQueue.get()
-        self.log.debug(f'Solve from queue [{mPoint["countSequence"]:03d}]: [{mPoint}]')
-
         self.app.showImage.emit(mPoint["imagePath"])
-
         self.resultQueue.put(mPoint)
         self.log.debug(f'Queued to result [{mPoint["countSequence"]:03d}]: [{mPoint}]')
         self.app.astrometry.solveThreading(fitsPath=mPoint["imagePath"],
-                                           updateFits=False,
-                                           )
-
+                                           updateFits=False)
         text = f'Solving  image-{mPoint["countSequence"]:03d}:  '
         text += f'path: {os.path.basename(mPoint["imagePath"])}'
         self.app.message.emit(text, 0)
@@ -299,16 +308,18 @@ class Model:
 
         :return: success
         """
-        self.log.debug('Imaging started')
-        if self.imageQueue.empty():
-            self.log.info('Empty image queue')
+        noImagesLeft = self.imageQueue.empty()
+        slewsLeft = not self.slewQueue.empty()
+        stillToWork = slewsLeft
+
+        if noImagesLeft and stillToWork:
+            t = f'Slews left: [{self.slewQueue.qsize()}] '
+            self.log.error(f'Empty image queue: {t}')
             return False
 
+        self.log.debug('Imaging started')
         mPoint = self.imageQueue.get()
-        self.log.debug(f'Image from queue [{mPoint["countSequence"]:03d}]: [{mPoint}]')
-
         self.collector.resetSignals()
-
         while self.ui.pauseModel.property('pause'):
             QTest.qWait(100)
 
@@ -317,8 +328,7 @@ class Model:
                                binning=mPoint['binning'],
                                subFrame=mPoint['subFrame'],
                                fastReadout=mPoint['fastReadout'],
-                               focalLength=mPoint['focalLength'],
-                               )
+                               focalLength=mPoint['focalLength'])
 
         mPoint['raJNowM'] = self.app.mount.obsSite.raJNow
         mPoint['decJNowM'] = self.app.mount.obsSite.decJNow
@@ -351,27 +361,21 @@ class Model:
         :return: success
 
         """
-        self.log.debug('Slew started')
         if self.slewQueue.empty():
-            self.log.info('Empty slew queue')
+            self.log.info('Empty slew queue- model sequence finished')
             return False
 
+        self.log.debug('Slew started')
         mPoint = self.slewQueue.get()
-        self.log.debug(f'Slew from queue [{mPoint["countSequence"]:03d}]: [{mPoint}]')
-
         suc = self.app.mount.obsSite.setTargetAltAz(alt_degrees=mPoint['altitude'],
-                                                    az_degrees=mPoint['azimuth'],
-                                                    )
+                                                    az_degrees=mPoint['azimuth'])
         if not suc:
             return False
 
         if self.deviceStat['dome']:
             alt = mPoint['altitude']
             az = mPoint['azimuth']
-
-            delta = self.app.dome.slewDome(altitude=alt,
-                                           azimuth=az)
-
+            delta = self.app.dome.slewDome(altitude=alt, azimuth=az)
             geoStat = 'Geometry corrected' if delta else 'Equal mount'
             text = f'Slewing  dome:       point: {mPoint["countSequence"]:03d}, '
             text += f'{geoStat}, az: {az:3.1f} delta: {delta:3.1f}'
@@ -381,7 +385,8 @@ class Model:
         self.imageQueue.put(mPoint)
         self.log.debug(f'Queued to image [{mPoint["countSequence"]:03d}]: [{mPoint}]')
 
-        text = f'Slewing  mount:      point: {mPoint["countSequence"]:03d}, '
+        text = 'Slewing  mount:      point in sequence: '
+        text += f'{mPoint["countSequence"]:03d}, '
         text += f'altitude: {mPoint["altitude"]:3.0f}, '
         text += f'azimuth: {mPoint["azimuth"]:3.0f}'
         self.app.message.emit(text, 0)
@@ -395,31 +400,23 @@ class Model:
         """
         :return: True for test purpose
         """
-        if not self.ui.checkDisableDAT.isChecked():
-            return False
-
         if self.statusDAT is None:
             self.statusDAT = self.app.mount.setting.statusDualAxisTracking
 
         self.statusDAT = self.app.mount.setting.statusDualAxisTracking
         self.app.mount.setting.setDualAxisTracking(False)
         self.changeStyleDynamic(self.ui.statusDualAxisTracking, 'color', 'yellow')
-
         return True
 
     def restoreStatusDAT(self):
         """
         :return: true for test purpose
         """
-        if not self.ui.checkDisableDAT.isChecked():
-            return False
-
         if self.statusDAT is None:
             return False
 
         self.app.mount.setting.setDualAxisTracking(self.statusDAT)
         self.changeStyleDynamic(self.ui.statusDualAxisTracking, 'color', '')
-
         return True
 
     def clearQueues(self):
@@ -448,6 +445,7 @@ class Model:
         self.ui.runFlexure.setEnabled(False)
         self.ui.runHysteresis.setEnabled(False)
         self.ui.batchModel.setEnabled(False)
+        self.app.enableEditPoints.emit(False)
 
         winImage = self.app.uiWindows['showImageW']['classObj']
         if not winImage:
@@ -490,6 +488,7 @@ class Model:
         self.ui.mImage.setText('-')
         self.ui.mSolve.setText('-')
         self.ui.modelProgress.setValue(0)
+        self.app.enableEditPoints.emit(True)
         return True
 
     def setupSignalsForModelRun(self):
@@ -512,7 +511,6 @@ class Model:
 
         if hasDome and hasAzimuth:
             self.collector.addWaitableSignal(self.app.dome.signals.slewFinished)
-
         elif hasDome and not hasAzimuth:
             self.app.message.emit('Dome without azimuth value used', 2)
 
@@ -542,9 +540,10 @@ class Model:
         """
         if self.ui.pauseModel.property('pause'):
             self.changeStyleDynamic(self.ui.pauseModel, 'pause', False)
-
+            self.ui.runModel.setEnabled(True)
         else:
             self.changeStyleDynamic(self.ui.pauseModel, 'pause', True)
+            self.ui.runModel.setEnabled(False)
 
         return True
 
@@ -562,7 +561,6 @@ class Model:
         self.clearQueues()
         self.restoreModelDefaultContextAndGuiStatus()
         self.app.message.emit('Modeling cancelled', 2)
-
         return True
 
     def retrofitModel(self):
@@ -583,7 +581,7 @@ class Model:
             self.log.debug(text)
             self.model = []
 
-        self.model = self.writeRetrofitData(mountModel, self.model)
+        self.model = writeRetrofitData(mountModel, self.model)
         return True
 
     def generateSaveModel(self):
@@ -723,7 +721,6 @@ class Model:
             self.app.data.setStatusBuildP(i, True)
 
         self.app.updatePointMarker.emit()
-
         return True
 
     def processModelData(self):
@@ -732,15 +729,14 @@ class Model:
         """
         suc = self.collectingModelRunOutput()
         if not suc:
-            self.app.message.emit(f'Modeling finished:    {self.modelName}', 2)
-            self.app.message.emit('Model not enough valid model points available', 2)
+            self.app.message.emit(f'Modeling error:       {self.modelName}', 2)
+            self.app.message.emit('Not enough valid model points available', 2)
             return False
 
         self.app.message.emit('Programming model to mount', 0)
         suc = self.programModelToMount(self.model)
         if suc:
             self.app.message.emit('Model programmed with success', 0)
-
         else:
             self.app.message.emit('Model programming error', 2)
 
@@ -757,7 +753,6 @@ class Model:
 
             if not suc:
                 self.app.message.emit('Cannot park mount', 2)
-
             else:
                 self.app.message.emit('Mount parked', 0)
 
@@ -778,25 +773,33 @@ class Model:
         if self.retryQueue.qsize() == 0:
             self.processModelData()
             return True
-
         if self.modelBuildRetryCounter == 0:
             self.processModelData()
             return True
 
         self.app.message.emit('Starting retry failed points', 1)
-        self.log.debug('Retry started')
+        maxRetries = self.ui.numberBuildRetries.value()
+        retryNumber = maxRetries - self.modelBuildRetryCounter + 1
+        self.app.message.emit(f'Retry run number: {retryNumber}', 1)
         numberPointsRetry = self.retryQueue.qsize()
         countPointsRetry = 0
 
+        points = list()
         while not self.retryQueue.empty():
-            point = self.retryQueue.get()
+            points.append(self.retryQueue.get())
+
+        if self.ui.retriesReverse.isChecked():
+            points = reversed(points)
+
+        for point in points:
             point['lenSequence'] = numberPointsRetry
-            point['countSequence'] = countPointsRetry
-            self.slewQueue.put(point)
+            point['countSequence'] = countPointsRetry + 1
             countPointsRetry += 1
+            self.slewQueue.put(point)
 
         self.modelBuildRetryCounter -= 1
         self.modelSlew()
+        self.ui.modelProgress.setValue(0)
         return True
 
     def checkModelRunConditions(self):
@@ -804,16 +807,19 @@ class Model:
         :return:
         """
         if len(self.app.data.buildP) < 2:
-            self.app.message.emit('No modeling start because less than 3 points', 2)
+            t = 'No modeling start because less than 3 points'
+            self.app.message.emit(t, 2)
             return False
 
         if len(self.app.data.buildP) > 99:
-            self.app.message.emit('No modeling start because more than 99 points', 2)
+            t = 'No modeling start because more than 99 points'
+            self.app.message.emit(t, 2)
             return False
 
         excludeDonePoints = self.ui.excludeDonePoints.isChecked()
         if len([x for x in self.app.data.buildP if x[2]]) < 3 and excludeDonePoints:
-            self.app.message.emit('No modeling start because less than 3 points left over', 2)
+            t = 'No modeling start because less than 3 points left over'
+            self.app.message.emit(t, 2)
             return False
 
         if self.ui.astrometryDevice.currentText().startswith('No device'):
@@ -836,7 +842,6 @@ class Model:
             self.app.message.emit('Actual model cannot be cleared', 2)
             self.app.message.emit('Model build cancelled', 2)
             return False
-
         else:
             self.app.message.emit('Actual model clearing, waiting 1s', 0)
             QTest.qWait(1000)
@@ -849,7 +854,8 @@ class Model:
 
         suc = self.app.mount.model.storeName('backup')
         if not suc:
-            self.app.message.emit('Cannot save backup model on mount, proceeding with model run', 2)
+            t = 'Cannot save backup model on mount, proceeding with model run'
+            self.app.message.emit(t, 2)
 
         return True
 
@@ -881,6 +887,7 @@ class Model:
             m['fastReadout'] = fastReadout
             m['lenSequence'] = lenSequence
             m['countSequence'] = index + 1
+            m['pointNumber'] = index + 1
             m['modelName'] = self.modelName
             m['imagePath'] = imagePath
             m['astrometryApp'] = astrometryApp
@@ -894,11 +901,11 @@ class Model:
 
     def modelCycleThroughBuildPoints(self, modelPoints=None):
         """
-        modelCycleThroughBuildPoints is the main method for preparing a model run. in
-        addition it checks necessary components and prepares all the parameters.
-        the modeling queue will be filled with point and the queue is started. the overall
-        modeling process consists of a set of queues which are handled by events running
-        in the gui event queue.
+        modelCycleThroughBuildPoints is the main method for preparing a model
+        run. in addition it checks necessary components and prepares all the
+        parameters. the modeling queue will be filled with point and the queue is
+        started. the overall modeling process consists of a set of queues which
+        are handled by events running in the gui event queue.
 
         :param modelPoints:
         :return: true for test purpose
@@ -909,6 +916,7 @@ class Model:
         for point in modelPoints:
             self.slewQueue.put(point)
 
+        self.app.dome.avoidFirstOvershoot()
         self.modelSlew()
         return True
 
@@ -934,7 +942,6 @@ class Model:
         """
         if not self.checkModelRunConditions():
             return False
-
         if not self.clearAlignAndBackup():
             return False
 
@@ -963,12 +970,11 @@ class Model:
         ret = self.openFile(self, 'Open model file', folder, 'Model files (*.model)',
                             multiple=True)
         loadFilePath, _, _ = ret
+
         if not loadFilePath:
             return False
-
         if isinstance(loadFilePath, str):
             loadFilePath = [loadFilePath]
-
         if not self.clearAlignAndBackup():
             return False
 
@@ -989,8 +995,150 @@ class Model:
 
         if suc:
             self.app.message.emit('Model programmed with success', 0)
-
         else:
             self.app.message.emit('Model programming error', 2)
 
         return suc
+
+    def syncMountAndClearUp(self):
+        """
+        :return:
+        """
+        self.ui.runModel.setEnabled(True)
+        self.ui.batchModel.setEnabled(True)
+        self.ui.plateSolveSync.setEnabled(True)
+        self.ui.runFlexure.setEnabled(True)
+        self.ui.runHysteresis.setEnabled(True)
+        return True
+
+    def solveDone(self, result=None):
+        """
+        :param result: result (named tuple)
+        :return: success
+        """
+        self.app.astrometry.signals.done.disconnect(self.solveDone)
+
+        if not result:
+            self.app.message.emit('Solving error, result missing', 2)
+            return False
+
+        if result['success']:
+            text = 'Solved :             '
+            text += f'RA: {convertToHMS(result["raJ2000S"])} '
+            text += f'({result["raJ2000S"].hours:4.3f}), '
+            text += f'DEC: {convertToDMS(result["decJ2000S"])} '
+            text += f'({result["decJ2000S"].degrees:4.3f}), '
+            self.app.message.emit(text, 0)
+            text = '                     '
+            text += f'Angle: {result["angleS"]:3.0f}, '
+            text += f'Scale: {result["scaleS"]:4.3f}, '
+            text += f'Error: {result["errorRMS_S"]:4.1f}'
+            self.app.message.emit(text, 0)
+
+        else:
+            text = f'Solving error:       {result.get("message")}'
+            self.app.message.emit(text, 2)
+            return False
+
+        self.app.showImage.emit(result['solvedPath'])
+
+        obs = self.app.mount.obsSite
+        timeJD = obs.timeJD
+        raJNow, decJNow = transform.J2000ToJNow(result['raJ2000S'],
+                                                result['decJ2000S'],
+                                                timeJD)
+        obs.setTargetRaDec(raJNow, decJNow)
+        suc = obs.syncPositionToTarget()
+        if suc:
+            t = 'Successfully synced model in mount to coordinates'
+            self.app.message.emit(t, 1)
+        else:
+            t = 'No sync, match failed because coordinates to far off for model'
+            self.app.message.emit(t, 2)
+        return suc
+
+    def solveImage(self, imagePath=''):
+        """
+        :param imagePath:
+        :return:
+        """
+        if not imagePath:
+            return False
+        if not os.path.isfile(imagePath):
+            return False
+
+        self.app.astrometry.signals.done.connect(self.solveDone)
+        self.app.astrometry.solveThreading(fitsPath=imagePath)
+        text = f'Solving:             [{os.path.basename(imagePath)}]'
+        self.app.message.emit(text, 0)
+        return True
+
+    def exposeRaw(self, expTime, binning, subFrame, fastReadout, focalLength):
+        """
+        :param expTime:
+        :param binning:
+        :param subFrame:
+        :param fastReadout:
+        :param focalLength:
+        :return: True for test purpose
+        """
+        time = self.app.mount.obsSite.timeJD.utc_strftime('%Y-%m-%d-%H-%M-%S')
+        fileName = time + '-sync.fits'
+        imagePath = self.app.mwGlob['imageDir'] + '/' + fileName
+
+        self.app.camera.expose(imagePath=imagePath,
+                               expTime=expTime,
+                               binning=binning,
+                               subFrame=subFrame,
+                               fastReadout=fastReadout,
+                               focalLength=focalLength
+                               )
+
+        text = f'Exposing:            [{os.path.basename(imagePath)}]'
+        self.app.message.emit(text, 0)
+        text = f'Duration:{expTime:3.0f}s  '
+        text += f'Bin:{binning:1.0f}  Sub:{subFrame:3.0f}%'
+        self.app.message.emit(f'                     {text}', 0)
+        return True
+
+    def exposeImageDone(self, imagePath=''):
+        """
+        :param imagePath:
+        :return: True for test purpose
+        """
+        self.app.camera.signals.saved.disconnect(self.exposeImageDone)
+        text = f'Exposed:             [{os.path.basename(imagePath)}]'
+        self.app.message.emit(text, 0)
+        self.solveImage(imagePath)
+        return True
+
+    def exposeImage(self):
+        """
+        :return: success
+        """
+        expTime = self.ui.expTime.value()
+        binning = self.ui.binning.value()
+        subFrame = self.ui.subFrame.value()
+        fastReadout = self.ui.checkFastDownload.isChecked()
+        focalLength = self.ui.focalLength.value()
+        self.app.camera.signals.saved.connect(self.exposeImageDone)
+        self.exposeRaw(expTime, binning, subFrame, fastReadout, focalLength)
+        return True
+
+    def plateSolveSync(self):
+        """
+        :return:
+        """
+        self.app.message.emit('Starting plate solve and sync model in mount', 1)
+        sucApp, sucIndex = self.app.astrometry.checkAvailability()
+        if not (sucApp and sucIndex):
+            self.app.message.emit('No valid configuration for plate solver', 2)
+            return False
+
+        self.ui.runModel.setEnabled(False)
+        self.ui.batchModel.setEnabled(False)
+        self.ui.plateSolveSync.setEnabled(False)
+        self.ui.runFlexure.setEnabled(False)
+        self.ui.runHysteresis.setEnabled(False)
+        self.exposeImage()
+        return True
