@@ -20,7 +20,7 @@ import datetime
 # external packages
 import PyQt5.QtCore
 import PyQt5.QtWidgets
-from skyfield.toposlib import Topos
+from skyfield.api import wgs84
 
 # local import
 from base import transform
@@ -40,11 +40,8 @@ class Mount(object):
         ms = self.app.mount.signals
         ms.locationDone.connect(self.updateLocGUI)
         ms.pointDone.connect(self.updatePointGUI)
-        ms.pointDone.connect(self.updateTimeGUI)
         ms.settingDone.connect(self.updateSettingGUI)
-        ms.settingDone.connect(self.updateSetStatGUI)
-        ms.settingDone.connect(self.updateSetSyncGUI)
-        ms.settingDone.connect(self.updateTrackingGui)
+        self.app.update1s.connect(self.showOffset)
 
         self.ui.park.clicked.connect(self.changePark)
         self.ui.flipMount.clicked.connect(self.flipMount)
@@ -53,6 +50,7 @@ class Mount(object):
         self.ui.setSiderealTracking.clicked.connect(self.setSiderealTracking)
         self.ui.setSolarTracking.clicked.connect(self.setSolarTracking)
         self.ui.stop.clicked.connect(self.stop)
+
         self.clickable(self.ui.meridianLimitTrack).connect(self.setMeridianLimitTrack)
         self.clickable(self.ui.meridianLimitSlew).connect(self.setMeridianLimitSlew)
         self.clickable(self.ui.horizonLimitHigh).connect(self.setHorizonLimitHigh)
@@ -65,13 +63,11 @@ class Mount(object):
         self.clickable(self.ui.statusDualAxisTracking).connect(self.setDualAxisTracking)
         self.clickable(self.ui.statusWOL).connect(self.setWOL)
         self.clickable(self.ui.statusRefraction).connect(self.setRefraction)
+        self.ui.virtualStop.raise_()
+        self.ui.virtualStop.clicked.connect(self.virtualStop)
 
     def initConfig(self):
         """
-        initConfig read the key out of the configuration dict and stores it to the gui
-        elements. if some initialisations have to be proceeded with the loaded persistent
-        data, they will be launched as well in this method.
-
         :return: True for test purpose
         """
         config = self.app.config.get('mainW', {})
@@ -82,10 +78,6 @@ class Mount(object):
 
     def storeConfig(self):
         """
-        storeConfig writes the keys to the configuration dict and stores. if some
-        saving has to be proceeded to persistent data, they will be launched as
-        well in this method.
-
         :return: True for test purpose
         """
         config = self.app.config['mainW']
@@ -98,198 +90,77 @@ class Mount(object):
         :param obs:
         :return:    True if ok for testing
         """
-        if obs.Alt is not None:
-            self.ui.ALT.setText('{0:5.2f}'.format(obs.Alt.degrees))
-
-        else:
-            self.ui.ALT.setText('-')
-
-        if obs.Az is not None:
-            self.ui.AZ.setText('{0:5.2f}'.format(obs.Az.degrees))
-
-        else:
-            self.ui.AZ.setText('-')
-
         isJ2000 = self.ui.checkJ2000.isChecked()
         isValid = obs.raJNow is not None
         isValid = isValid and obs.decJNow is not None
         isValid = isValid and obs.timeJD is not None
         if isJ2000 and isValid:
             ra, dec = transform.JNowToJ2000(obs.raJNow, obs.decJNow, obs.timeJD)
-
         else:
             ra = obs.raJNow
             dec = obs.decJNow
 
-        if ra is not None:
-            self.ui.RA.setText(self.formatHstrToText(ra))
-            self.ui.RAfloat.setText(f'{ra.hours:3.4f}')
-
-        else:
-            self.ui.RA.setText('-')
-            self.ui.RAfloat.setText('-')
-
-        if dec is not None:
-            self.ui.DEC.setText(self.formatDstrToText(dec))
-            self.ui.DECfloat.setText(f'{dec.degrees:+3.4f}')
-
-        else:
-            self.ui.DEC.setText('-')
-            self.ui.DECfloat.setText('-')
-
-        if obs.pierside is not None:
-            self.ui.pierside.setText('WEST' if obs.pierside == 'W' else 'EAST')
-
-        else:
-            self.ui.pierside.setText('-')
-
-        if obs.haJNow is not None:
-            self.ui.HA.setText(self.formatHstrToText(obs.haJNow))
-            self.ui.HAfloat.setText(f'{obs.haJNow.hours:3.4f}')
-
-        else:
-            self.ui.HA.setText('-')
-            self.ui.HAfloat.setText('-')
-
+        self.guiSetText(self.ui.RA, 'HSTR', ra)
+        self.guiSetText(self.ui.RAfloat, 'H5.4f', ra)
+        self.guiSetText(self.ui.DEC, 'DSTR', dec)
+        self.guiSetText(self.ui.DECfloat, 'D5.4f', dec)
+        self.guiSetText(self.ui.HA, 'HSTR', obs.haJNow)
+        self.guiSetText(self.ui.HAfloat, 'H5.4f', obs.haJNow)
+        self.guiSetText(self.ui.ALT, 'D5.2f', obs.Alt)
+        self.guiSetText(self.ui.AZ, 'D5.2f', obs.Az)
+        self.guiSetText(self.ui.pierside, 's', obs.pierside)
+        self.guiSetText(self.ui.timeSidereal, 'HSTR', obs.timeSidereal)
         return True
 
-    def updateTimeGUI(self, obs):
-        """
-        :param obs:
-        :return:    True if ok for testing
-        """
-        if obs.timeJD is not None:
-            text = obs.timeJD.utc_strftime('%H:%M:%S')
-            self.ui.timeUTC.setText('UTC: ' + text)
-
-        if obs.timeSidereal is not None:
-            siderealFormat = '{0:02.0f}:{1:02.0f}:{2:02.0f}'
-            val = obs.timeSidereal.hms()
-            siderealText = siderealFormat.format(*val)
-            self.ui.timeSidereal.setText(siderealText)
-
-        else:
-            self.ui.timeSidereal.setText('-')
-
-        return True
-
-    def updateSetStatGUI(self, sett):
+    def updateSettingGUI(self, sett):
         """
         :param sett:
         :return:    True if ok for testing
         """
+        ui = self.ui.UTCExpire
+        self.guiSetText(ui, 's', sett.UTCExpire)
         if sett.UTCExpire is not None:
-            ui = self.ui.UTCExpire
-            ui.setText(sett.UTCExpire)
-
-            # coloring if close to end:
             now = datetime.datetime.now()
             expire = datetime.datetime.strptime(sett.UTCExpire, '%Y-%m-%d')
             deltaYellow = datetime.timedelta(days=30)
-
             if now > expire:
+                self.changeStyleDynamic(ui, 'char', 'red')
                 self.changeStyleDynamic(ui, 'color', 'red')
-
             elif now > expire - deltaYellow:
-                self.changeStyleDynamic(ui, 'color', 'yellow')
-
+                self.changeStyleDynamic(ui, 'char', '')
+                self.changeStyleDynamic(ui, 'color', 'red')
             else:
                 self.changeStyleDynamic(ui, 'color', '')
-
-        self.guiSetText(self.ui.UTCExpire, 's', sett.UTCExpire)
+                self.changeStyleDynamic(ui, 'char', '')
 
         ui = self.ui.statusUnattendedFlip
-        if sett.statusUnattendedFlip is None:
-            text = '-'
-            self.changeStyleDynamic(ui, 'status', '')
-
-        elif sett.statusUnattendedFlip:
-            text = 'ON'
-            self.changeStyleDynamic(ui, 'status', 'on')
-
-        else:
-            text = 'OFF'
-            self.changeStyleDynamic(ui, 'status', '')
-
-        self.guiSetText(ui, 's', text)
+        self.guiSetText(ui, 's', sett.statusUnattendedFlip)
+        self.guiSetStyle(ui, 'status', sett.statusUnattendedFlip, ['', 'on', ''])
 
         ui = self.ui.statusDualAxisTracking
-        if sett.statusDualAxisTracking is None:
-            text = '-'
-            self.changeStyleDynamic(ui, 'status', '')
-
-        elif sett.statusDualAxisTracking:
-            text = 'ON'
-            self.changeStyleDynamic(ui, 'status', 'on')
-
-        else:
-            text = 'OFF'
-            self.changeStyleDynamic(ui, 'status', '')
-
-        self.guiSetText(ui, 's', text)
+        self.guiSetText(ui, 's', sett.statusDualAxisTracking)
+        self.guiSetStyle(ui, 'status', sett.statusDualAxisTracking, ['', 'on', ''])
 
         ui = self.ui.statusRefraction
-        if sett.statusRefraction is None:
-            text = '-'
-            self.changeStyleDynamic(ui, 'status', '')
-            self.changeStyleDynamic(self.ui.refractionTemp1, 'color', '')
-            self.changeStyleDynamic(self.ui.refractionPress1, 'color', '')
+        self.guiSetText(ui, 's', sett.statusRefraction)
+        self.guiSetStyle(ui, 'status', sett.statusRefraction, ['', 'on', ''])
 
-        elif sett.statusRefraction:
-            text = 'ON'
-            self.changeStyleDynamic(ui, 'status', 'on')
-            self.changeStyleDynamic(self.ui.refractionTemp1, 'color', '')
-            self.changeStyleDynamic(self.ui.refractionPress1, 'color', '')
-
-        else:
-            text = 'OFF'
-            self.changeStyleDynamic(ui, 'status', '')
-            self.changeStyleDynamic(self.ui.refractionTemp1, 'color', 'yellow')
-            self.changeStyleDynamic(self.ui.refractionPress1, 'color', 'yellow')
-
-        self.guiSetText(ui, 's', text)
-
-        return True
-
-    def updateSetSyncGUI(self, sett):
-        """
-        :param sett:
-        :return:    True if ok for testing
-        """
         ui = self.ui.statusGPSSynced
-        if sett.gpsSynced is None:
-            text = '-'
-            self.changeStyleDynamic(ui, 'status', '')
-        elif sett.gpsSynced:
-            text = 'YES'
-            self.changeStyleDynamic(ui, 'status', 'on')
-        else:
-            text = 'NO'
-            self.changeStyleDynamic(ui, 'status', '')
-        self.guiSetText(ui, 's', text)
+        self.guiSetText(ui, 's', sett.gpsSynced)
+        self.guiSetStyle(ui, 'status', sett.gpsSynced, ['', 'on', ''])
 
         ui = self.ui.statusWOL
-        if sett.wakeOnLan is None:
-            text = '-'
-            self.changeStyleDynamic(ui, 'status', '')
-        elif sett.wakeOnLan == 'On':
-            text = 'ON'
-            self.changeStyleDynamic(ui, 'status', 'on')
-        else:
-            text = 'OFF'
-            self.changeStyleDynamic(ui, 'status', '')
-        self.guiSetText(ui, 's', text)
+        self.guiSetText(ui, 's', sett.wakeOnLan)
+        self.guiSetStyle(ui, 'status', sett.wakeOnLan, ['', 'on', ''])
+
+        ui = self.ui.statusWebInterface
+        self.guiSetText(ui, 's', sett.webInterfaceStat)
+        self.guiSetStyle(ui, 'status', sett.webInterfaceStat, ['', 'on', ''])
 
         if sett.typeConnection is not None:
             text = self.typeConnectionTexts[sett.typeConnection]
             self.ui.mountTypeConnection.setText(text)
 
-        return True
-
-    def updateSettingGUI(self, sett):
-        """
-        :return:    True if ok for testing
-        """
         self.guiSetText(self.ui.slewRate, '2.0f', sett.slewRate)
         self.guiSetText(self.ui.timeToFlip, '3.0f', sett.timeToFlip)
         self.guiSetText(self.ui.timeToMeridian, '3.0f', sett.timeToMeridian())
@@ -302,37 +173,13 @@ class Mount(object):
         self.guiSetText(self.ui.horizonLimitLow, '3.0f', sett.horizonLimitLow)
         self.guiSetText(self.ui.horizonLimitHigh, '3.0f', sett.horizonLimitHigh)
 
-        return True
-
-    def updateLocGUI(self, obs):
-        """
-        :param obs:
-        :return:    True if ok for testing
-        """
-        if obs is None:
-            return False
-
-        location = obs.location
-        if location is None:
-            return False
-
-        self.ui.siteLongitude.setText(self.formatLonToText(location.longitude))
-        self.ui.siteLatitude.setText(self.formatLatToText(location.latitude))
-        self.ui.siteElevation.setText(str(location.elevation.m))
-
-        return True
-
-    def updateTrackingGui(self, sett):
-        """
-        :param sett:
-        :return:    True if ok for testing
-        """
-        if sett is None:
-            return False
         if self.app.mount.obsSite.status is None:
-            return False
+            self.changeStyleDynamic(self.ui.followSat, 'running', 'false')
+            self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'false')
+            self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'false')
+            self.changeStyleDynamic(self.ui.setSolarTracking, 'running', 'false')
 
-        if self.app.mount.obsSite.status == 10:
+        elif self.app.mount.obsSite.status == 10:
             self.changeStyleDynamic(self.ui.followSat, 'running', 'true')
             self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'false')
             self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'false')
@@ -355,13 +202,33 @@ class Mount(object):
             self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'false')
             self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'false')
             self.changeStyleDynamic(self.ui.setSolarTracking, 'running', 'true')
+        return True
 
+    def updateLocGUI(self, obs):
+        """
+        :param obs:
+        :return:    True if ok for testing
+        """
+        if obs is None:
+            return False
+        location = obs.location
+        if location is None:
+            return False
+
+        self.ui.siteLongitude.setText(self.formatLonToText(location.longitude))
+        self.ui.siteLatitude.setText(self.formatLatToText(location.latitude))
+        self.ui.siteElevation.setText(str(location.elevation.m))
         return True
 
     def changeTracking(self):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         if obs.status == 0:
             suc = obs.stopTracking()
@@ -383,6 +250,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         if obs.status == 5:
             suc = obs.unpark()
@@ -404,6 +276,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
+            return False
+
         sett = self.app.mount.setting
         suc = sett.setLunarTracking()
         if not suc:
@@ -418,6 +295,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
+            return False
+
         sett = self.app.mount.setting
         suc = sett.setSiderealTracking()
         if not suc:
@@ -432,6 +314,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
+            return False
+
         sett = self.app.mount.setting
         suc = sett.setSolarTracking()
         if not suc:
@@ -446,6 +333,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         suc = obs.flip()
         if not suc:
@@ -460,6 +352,11 @@ class Mount(object):
         """
         :return:
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         suc = obs.stop()
         if not suc:
@@ -470,12 +367,21 @@ class Mount(object):
             self.app.message.emit('Mount stopped', 0)
             return True
 
+    def virtualStop(self):
+        """
+        :return:
+        """
+        if self.ui.activateVirtualStop.isChecked():
+            self.stop()
+
     def setMeridianLimitTrack(self):
         """
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -496,11 +402,9 @@ class Mount(object):
 
         if not ok:
             return False
-
         if sett.setMeridianLimitTrack(value):
             self.app.message.emit(f'Meridian Limit Track: [{value}]', 0)
             return True
-
         else:
             self.app.message.emit('Meridian Limit Track cannot be set', 2)
             return False
@@ -510,7 +414,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -531,11 +437,9 @@ class Mount(object):
 
         if not ok:
             return False
-
         if sett.setMeridianLimitSlew(value):
             self.app.message.emit(f'Meridian Limit Slew: [{value}]', 0)
             return True
-
         else:
             self.app.message.emit('Meridian Limit Slew cannot be set', 2)
             return False
@@ -545,7 +449,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -566,11 +472,9 @@ class Mount(object):
 
         if not ok:
             return False
-
         if sett.setHorizonLimitHigh(value):
             self.app.message.emit(f'Horizon Limit High: [{value}]', 0)
             return True
-
         else:
             self.app.message.emit('Horizon Limit High cannot be set', 2)
             return False
@@ -580,7 +484,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -601,11 +507,9 @@ class Mount(object):
 
         if not ok:
             return False
-
         if sett.setHorizonLimitLow(value):
             self.app.message.emit(f'Horizon Limit Low: [{value}]', 0)
             return True
-
         else:
             self.app.message.emit('Horizon Limit Low cannot be set', 2)
             return False
@@ -615,7 +519,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -638,11 +544,9 @@ class Mount(object):
 
         if not ok:
             return False
-
         if sett.setSlewRate(value):
             self.app.message.emit(f'Slew Rate: [{value}]', 0)
             return True
-
         else:
             self.app.message.emit('Slew Rate cannot be set', 2)
             return False
@@ -651,6 +555,11 @@ class Mount(object):
         """
         :return:    success as bool if value could be changed
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         if obs.location is None:
             return False
@@ -667,25 +576,19 @@ class Mount(object):
             return False
 
         value = self.convertLonToAngle(value)
-
         if value is None:
             return False
 
-        topo = Topos(longitude=value,
-                     latitude=obs.location.latitude,
-                     elevation_m=obs.location.elevation.m)
+        topo = wgs84.latlon(longitude_degrees=value.degrees,
+                            latitude_degrees=obs.location.latitude.degrees,
+                            elevation_m=obs.location.elevation.m)
         obs.location = topo
 
-        if not self.deviceStat.get('mount', ''):
-            self.updateLocGUI(obs)
-            return False
-
         if obs.setLongitude(value):
             self.app.message.emit(f'Longitude set to:    '
-                                  f'[{self.ui.siteLongitude.text()}]', 0)
+                                  f'[{value}]', 0)
             self.app.mount.getLocation()
             return True
-
         else:
             self.app.message.emit('Longitude cannot be set', 2)
             return False
@@ -694,6 +597,11 @@ class Mount(object):
         """
         :return:    success as bool if value could be changed
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
         if obs.location is None:
             return False
@@ -712,21 +620,16 @@ class Mount(object):
         if value is None:
             return False
 
-        topo = Topos(longitude=obs.location.longitude,
-                     latitude=value,
-                     elevation_m=obs.location.elevation.m)
+        topo = wgs84.latlon(longitude_degrees=obs.location.longitude.degrees,
+                            latitude_degrees=value.degrees,
+                            elevation_m=obs.location.elevation.m)
         obs.location = topo
 
-        if not self.deviceStat.get('mount', ''):
-            self.updateLocGUI(obs)
-            return False
-
         if obs.setLatitude(value):
             self.app.message.emit(f'Latitude set to:     '
-                                  f'[{self.ui.siteLatitude.text()}]', 0)
+                                  f'[{value}]', 0)
             self.app.mount.getLocation()
             return True
-
         else:
             self.app.message.emit('Latitude cannot be set', 2)
             return False
@@ -735,6 +638,11 @@ class Mount(object):
         """
         :return:    success as bool if value could be changed
         """
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.obsSite is not None
+        if not isMount or not isData:
+            return False
+
         obs = self.app.mount.obsSite
 
         if obs.location is None:
@@ -752,29 +660,27 @@ class Mount(object):
         if not ok:
             return False
 
-        topo = Topos(longitude=obs.location.longitude,
-                     latitude=obs.location.latitude,
-                     elevation_m=value)
+        topo = wgs84.latlon(longitude_degrees=obs.location.longitude.degrees,
+                            latitude_degrees=obs.location.latitude.degrees,
+                            elevation_m=value)
         obs.location = topo
 
-        if not self.deviceStat.get('mount', ''):
-            self.updateLocGUI(obs)
-            return False
-
         if obs.setElevation(value):
             self.app.message.emit(f'Elevation set to:    [{value}]', 0)
             self.app.mount.getLocation()
             return True
-
         else:
             self.app.message.emit('Elevation cannot be set', 2)
+            return False
 
     def setUnattendedFlip(self):
         """
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -792,11 +698,9 @@ class Mount(object):
                                 )
         if not ok:
             return False
-
         suc = sett.setUnattendedFlip(value == 'ON')
         if suc:
             self.app.message.emit(f'Unattended flip set to [{value}]', 0)
-
         else:
             self.app.message.emit('Unattended flip cannot be set', 2)
         return suc
@@ -806,7 +710,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -828,7 +734,6 @@ class Mount(object):
         suc = sett.setDualAxisTracking(value == 'ON')
         if suc:
             self.app.message.emit(f'Dual axis tracking set to [{value}]', 0)
-
         else:
             self.app.message.emit('Dual axis tracking cannot be set', 2)
         return suc
@@ -838,7 +743,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -860,7 +767,6 @@ class Mount(object):
         suc = sett.setWOL(value == 'ON')
         if suc:
             self.app.message.emit(f'WOL set to [{value}]', 0)
-
         else:
             self.app.message.emit('WOL cannot be set', 2)
         return suc
@@ -870,7 +776,9 @@ class Mount(object):
         :return:    success as bool if value could be changed
         """
         msg = PyQt5.QtWidgets.QMessageBox
-        if not self.deviceStat.get('mount', ''):
+        isMount = self.deviceStat.get('mount', False)
+        isData = self.app.mount.setting is not None
+        if not isMount or not isData:
             msg.critical(self,
                          'Error Message',
                          'Value cannot be set when mount not connected !')
@@ -892,7 +800,33 @@ class Mount(object):
         suc = sett.setRefraction(value == 'ON')
         if suc:
             self.app.message.emit(f'Refraction correction set to [{value}]', 0)
-
         else:
             self.app.message.emit('Refraction correction cannot be set', 2)
         return suc
+
+    def showOffset(self):
+        """
+        :return:
+        """
+        connectSync = self.ui.clockSync.isChecked()
+        delta = self.app.mount.obsSite.timeDiff * 1000
+        ui = self.ui.timeDeltaPC2Mount
+        if connectSync:
+            text = f'{delta:4.0f}'
+        else:
+            text = '-'
+        ui.setText(text)
+
+        if not connectSync:
+            self.changeStyleDynamic(ui, 'char', '')
+            self.changeStyleDynamic(ui, 'color', '')
+        elif abs(delta) < 100:
+            self.changeStyleDynamic(ui, 'char', '')
+            self.changeStyleDynamic(ui, 'color', '')
+        elif abs(delta) < 500:
+            self.changeStyleDynamic(ui, 'char', 'yellow')
+            self.changeStyleDynamic(ui, 'color', '')
+        else:
+            self.changeStyleDynamic(ui, 'char', 'red')
+            self.changeStyleDynamic(ui, 'color', 'red')
+        return True
