@@ -38,11 +38,16 @@ class SettImaging(object):
         self.ui.coolerOn.clicked.connect(self.setCoolerOn)
         self.ui.coolerOff.clicked.connect(self.setCoolerOff)
         self.clickable(self.ui.coolerTemp).connect(self.setCoolerTemp)
+        self.clickable(self.ui.gainCam).connect(self.setGain)
+        self.clickable(self.ui.offsetCam).connect(self.setOffset)
         self.clickable(self.ui.filterNumber).connect(self.setFilterNumber)
         self.clickable(self.ui.filterName).connect(self.setFilterName)
         self.ui.coverPark.clicked.connect(self.setCoverPark)
         self.ui.coverUnpark.clicked.connect(self.setCoverUnpark)
         self.ui.coverHalt.clicked.connect(self.setCoverHalt)
+        self.ui.coverLightOn.clicked.connect(self.switchLightOn)
+        self.ui.coverLightOff.clicked.connect(self.switchLightOff)
+        self.clickable(self.ui.coverLightIntensity).connect(self.setLightIntensity)
         self.ui.copyFromTelescopeDriver.clicked.connect(self.updateTelescopeParametersToGui)
         self.ui.aperture.valueChanged.connect(self.updateParameters)
         self.ui.focalLength.valueChanged.connect(self.updateParameters)
@@ -55,14 +60,13 @@ class SettImaging(object):
         self.ui.moveFocuserIn.clicked.connect(self.moveFocuserIn)
         self.ui.moveFocuserOut.clicked.connect(self.moveFocuserOut)
         self.app.update1s.connect(self.updateCoverStatGui)
+        self.app.update1s.connect(self.updateCoverLightGui)
         self.app.update1s.connect(self.updateParameters)
 
     def initConfig(self):
         """
-
         :return:
         """
-
         config = self.app.config['mainW']
         self.ui.expTime.setValue(config.get('expTime', 1))
         self.ui.binning.setValue(config.get('binning', 1))
@@ -75,15 +79,12 @@ class SettImaging(object):
         self.ui.checkFastDownload.setChecked(config.get('checkFastDownload', False))
         self.ui.checkKeepImages.setChecked(config.get('checkKeepImages', False))
         self.ui.checkAutomaticTelescope.setChecked(config.get('checkAutomaticTelescope', False))
-
         return True
 
     def storeConfig(self):
         """
-
         :return:
         """
-
         config = self.app.config['mainW']
         config['expTime'] = self.ui.expTime.value()
         config['binning'] = self.ui.binning.value()
@@ -96,7 +97,6 @@ class SettImaging(object):
         config['checkFastDownload'] = self.ui.checkFastDownload.isChecked()
         config['checkKeepImages'] = self.ui.checkKeepImages.isChecked()
         config['checkAutomaticTelescope'] = self.ui.checkAutomaticTelescope.isChecked()
-
         return True
 
     def updateParameters(self):
@@ -106,7 +106,6 @@ class SettImaging(object):
 
         :return: true for test purpose
         """
-
         if self.ui.checkAutomaticTelescope.isChecked():
             self.updateTelescopeParametersToGui()
 
@@ -121,6 +120,9 @@ class SettImaging(object):
         rotation = self.app.camera.data.get('CCD_ROTATION.CCD_ROTATION_VALUE', 0)
         coolerTemp = self.app.camera.data.get('CCD_TEMPERATURE.CCD_TEMPERATURE_VALUE', 0)
         coolerPower = self.app.camera.data.get('CCD_COOLER_POWER.CCD_COOLER_VALUE', 0)
+        gainCam = self.app.camera.data.get('CCD_GAIN.GAIN')
+        offsetCam = self.app.camera.data.get('CCD_OFFSET.OFFSET')
+        humidityCCD = self.app.camera.data.get('CCD_HUMIDITY.HUMIDITY')
         coolerOn = self.app.camera.data.get('CCD_COOLER.COOLER_ON', False)
         downloadFast = self.app.camera.data.get('READOUT_QUALITY.QUALITY_LOW', False)
         focus = self.app.focuser.data.get('ABS_FOCUS_POSITION.FOCUS_ABSOLUTE_POSITION', 0)
@@ -146,14 +148,12 @@ class SettImaging(object):
         if focalLength and pixelSizeX and pixelSizeY:
             resolutionX = pixelSizeX / focalLength * 206.265
             resolutionY = pixelSizeY / focalLength * 206.265
-
         else:
             resolutionX = None
             resolutionY = None
 
         if aperture:
             speed = focalLength / aperture
-
         else:
             speed = None
 
@@ -161,7 +161,6 @@ class SettImaging(object):
             dawes = 116 / aperture
             rayleigh = 138 / aperture
             magLimit = 7.7 + (5 * np.log10(aperture / 10))
-
         else:
             dawes = None
             rayleigh = None
@@ -170,7 +169,6 @@ class SettImaging(object):
         if focalLength and pixelSizeY and pixelSizeY and pixelX and pixelY:
             FOVX = pixelSizeX / focalLength * 206.265 * pixelX / 3600
             FOVY = pixelSizeY / focalLength * 206.265 * pixelY / 3600
-
         else:
             FOVX = None
             FOVY = None
@@ -183,8 +181,11 @@ class SettImaging(object):
         self.guiSetText(self.ui.pixelX, '5.0f', pixelX)
         self.guiSetText(self.ui.pixelY, '5.0f', pixelY)
         self.guiSetText(self.ui.rotation, '3.1f', rotation)
+        self.guiSetText(self.ui.humidityCCD, '3.2f', humidityCCD)
         self.guiSetText(self.ui.coolerTemp, '3.1f', coolerTemp)
         self.guiSetText(self.ui.coolerPower, '3.1f', coolerPower)
+        self.guiSetText(self.ui.gainCam, '3.0f', gainCam)
+        self.guiSetText(self.ui.offsetCam, '3.0f', offsetCam)
         self.guiSetText(self.ui.focuserPosition, '6.0f', focus)
         self.guiSetText(self.ui.resolutionX, '2.2f', resolutionX)
         self.guiSetText(self.ui.resolutionY, '2.2f', resolutionY)
@@ -197,7 +198,6 @@ class SettImaging(object):
         if coolerOn:
             self.changeStyleDynamic(self.ui.coolerOn, 'running', True)
             self.changeStyleDynamic(self.ui.coolerOff, 'running', False)
-
         else:
             self.changeStyleDynamic(self.ui.coolerOn, 'running', False)
             self.changeStyleDynamic(self.ui.coolerOff, 'running', True)
@@ -205,7 +205,6 @@ class SettImaging(object):
         if downloadFast:
             self.changeStyleDynamic(self.ui.downloadFast, 'running', True)
             self.changeStyleDynamic(self.ui.downloadSlow, 'running', False)
-
         else:
             self.changeStyleDynamic(self.ui.downloadFast, 'running', False)
             self.changeStyleDynamic(self.ui.downloadSlow, 'running', True)
@@ -214,8 +213,8 @@ class SettImaging(object):
 
     def updateTelescopeParametersToGui(self):
         """
-        updateTelescopeParametersToGui takes the information gathered from the driver and
-        programs them into gui for later use.
+        updateTelescopeParametersToGui takes the information gathered from the
+        driver and programs them into gui for later use.
 
         :return: true for test purpose
         """
@@ -233,9 +232,6 @@ class SettImaging(object):
 
     def setCoolerTemp(self):
         """
-        setCoolerTemp sends the desired cooler temp and switches the cooler on.
-        setting
-
         :return: success
         """
         canSetCCDTemp = self.app.camera.data.get('CAN_SET_CCD_TEMPERATURE', False)
@@ -243,8 +239,8 @@ class SettImaging(object):
             return False
 
         msg = PyQt5.QtWidgets.QMessageBox
-        actValue = self.app.camera.data.get('CCD_TEMPERATURE.CCD_TEMPERATURE_VALUE')
-
+        actValue = self.app.camera.data.get(
+            'CCD_TEMPERATURE.CCD_TEMPERATURE_VALUE', None)
         if actValue is None:
             msg.critical(self,
                          'Error Message',
@@ -259,18 +255,68 @@ class SettImaging(object):
                                20,
                                1,
                                )
-
         if not ok:
             return False
 
         self.app.camera.sendCoolerTemp(temperature=value)
         return True
 
+    def setOffset(self):
+        """
+        :return: success
+        """
+        msg = PyQt5.QtWidgets.QMessageBox
+        actValue = self.app.camera.data.get('CCD_OFFSET.OFFSET', None)
+        if actValue is None:
+            msg.critical(self,
+                         'Error Message',
+                         'Value cannot be set when not connected !')
+            return False
+        dlg = PyQt5.QtWidgets.QInputDialog()
+        value, ok = dlg.getInt(self,
+                               'Set offset',
+                               'Value (0..255):',
+                               actValue,
+                               0,
+                               255,
+                               1,
+                               )
+        if not ok:
+            return False
+
+        self.app.camera.sendOffset(offset=value)
+        return True
+
+    def setGain(self):
+        """
+        :return: success
+        """
+        msg = PyQt5.QtWidgets.QMessageBox
+        minGain = self.app.camera.data.get('CCD_INFO.GAIN_MIN', 1)
+        maxGain = self.app.camera.data.get('CCD_INFO.GAIN_MAX', 200)
+        actValue = self.app.camera.data.get('CCD_GAIN.GAIN', None)
+        if actValue is None:
+            msg.critical(self,
+                         'Error Message',
+                         'Value cannot be set when not connected !')
+            return False
+        dlg = PyQt5.QtWidgets.QInputDialog()
+        value, ok = dlg.getInt(self,
+                               'Set gain',
+                               'Value (0..255):',
+                               actValue,
+                               minGain,
+                               maxGain,
+                               1,
+                               )
+        if not ok:
+            return False
+
+        self.app.camera.sendGain(gain=value)
+        return True
+
     def setFilterNumber(self):
         """
-        setFilterNumber sends the desired filter number.
-        setting
-
         :return: success
         """
         msg = PyQt5.QtWidgets.QMessageBox
@@ -283,15 +329,12 @@ class SettImaging(object):
                          'Value cannot be set when not connected !')
             return False
 
-        isAlpaca = 'FILTER_NAME.FILTER_SLOT_NAME_0' in data
-
         availNames = list(data[key] for key in data if 'FILTER_NAME.FILTER_SLOT_NAME_' in key)
         numberFilter = len(availNames)
-
+        isAlpaca = 'FILTER_NAME.FILTER_SLOT_NAME_0' in data
         if isAlpaca:
             start = 0
             end = numberFilter - 1
-
         else:
             start = 1
             end = numberFilter
@@ -314,9 +357,6 @@ class SettImaging(object):
 
     def setFilterName(self):
         """
-        setFilterName sends the desired filter name
-        setting
-
         :return: success
         """
         msg = PyQt5.QtWidgets.QMessageBox
@@ -338,14 +378,13 @@ class SettImaging(object):
                                 availNames,
                                 actValue - 1,
                                 )
-
+        self.log.debug(f'FilterSelected: [{value}], FilterList: [{availNames}]')
         if not ok:
             return False
 
         isAlpaca = 'FILTER_NAME.FILTER_SLOT_NAME_0' in data
         if isAlpaca:
             number = availNames.index(value)
-
         else:
             number = availNames.index(value) + 1
 
@@ -370,10 +409,6 @@ class SettImaging(object):
         """
         :return:
         """
-        canGetCoolerPower = self.app.camera.data.get('CAN_GET_COOLER_POWER', False)
-        if not canGetCoolerPower:
-            return False
-
         self.app.camera.sendCoolerSwitch(coolerOn=True)
         return True
 
@@ -381,35 +416,48 @@ class SettImaging(object):
         """
         :return:
         """
-        canGetCoolerPower = self.app.camera.data.get('CAN_GET_COOLER_POWER', False)
-        if not canGetCoolerPower:
-            return False
-
         self.app.camera.sendCoolerSwitch(coolerOn=False)
         return True
 
     def updateCoverStatGui(self):
         """
-        updateCoverStatGui changes the style of the button related to the state of the
-        FlipFlat cover
-
         :return: True for test purpose
         """
-        value = self.app.cover.data.get('CAP_PARK.UNPARK', None)
-        if value == 'On':
-            self.changeStyleDynamic(self.ui.coverUnpark, 'running', True)
-            self.changeStyleDynamic(self.ui.coverPark, 'running', False)
-        elif value == 'Off':
+        value = self.app.cover.data.get('CAP_PARK.PARK', None)
+        if value:
             self.changeStyleDynamic(self.ui.coverPark, 'running', True)
             self.changeStyleDynamic(self.ui.coverUnpark, 'running', False)
+        elif value is None:
+            self.changeStyleDynamic(self.ui.coverPark, 'running', False)
+            self.changeStyleDynamic(self.ui.coverUnpark, 'running', False)
         else:
             self.changeStyleDynamic(self.ui.coverPark, 'running', False)
-            self.changeStyleDynamic(self.ui.coverUnpark, 'running', False)
+            self.changeStyleDynamic(self.ui.coverUnpark, 'running', True)
 
         value = self.app.cover.data.get('Status.Cover', '-')
         self.ui.coverStatusText.setText(value)
         return True
 
+    def updateCoverLightGui(self):
+        """
+        :return: True for test purpose
+        """
+        value = self.app.cover.data.get('FLAT_LIGHT_CONTROL.FLAT_LIGHT_ON', None)
+        if value:
+            self.changeStyleDynamic(self.ui.coverLightOn, 'running', True)
+            self.changeStyleDynamic(self.ui.coverLightOff, 'running', False)
+        elif value is None:
+            self.changeStyleDynamic(self.ui.coverLightOn, 'running', False)
+            self.changeStyleDynamic(self.ui.coverLightOff, 'running', False)
+        else:
+            self.changeStyleDynamic(self.ui.coverLightOn, 'running', False)
+            self.changeStyleDynamic(self.ui.coverLightOff, 'running', True)
+
+        value = self.app.cover.data.get(
+            'FLAT_LIGHT_INTENSITY.FLAT_LIGHT_INTENSITY_VALUE')
+        self.guiSetText(self.ui.coverLightIntensity, '3.0f', value)
+        return True
+
     def setCoverPark(self):
         """
         :return: success
@@ -467,5 +515,54 @@ class SettImaging(object):
         """
         suc = self.app.focuser.halt()
         if not suc:
-            self.app.message.emit('Focuser halt could not be executed', 2)
+            self.app.message.emit('Light could not be switched on', 2)
+        return suc
+
+    def switchLightOn(self):
+        """
+        :return:
+        """
+        suc = self.app.cover.lightOn()
+        if not suc:
+            self.app.message.emit('Light could not be switched on', 2)
+        return suc
+
+    def switchLightOff(self):
+        """
+        :return:
+        """
+        suc = self.app.cover.lightOff()
+        if not suc:
+            self.app.message.emit('Light could not be switched off', 2)
+        return suc
+
+    def setLightIntensity(self):
+        """
+        :return: success
+        """
+        msg = PyQt5.QtWidgets.QMessageBox
+        actValue = self.app.cover.data.get(
+            'FLAT_LIGHT_INTENSITY.FLAT_LIGHT_INTENSITY_VALUE')
+        if actValue is None:
+            msg.critical(self,
+                         'Error Message',
+                         'Value cannot be set when not connected !')
+            return False
+        dlg = PyQt5.QtWidgets.QInputDialog()
+        value, ok = dlg.getInt(self,
+                               'Set light intensity',
+                               'Value (0..255):',
+                               float(actValue),
+                               0,
+                               255,
+                               1,
+                               )
+        if not ok:
+            return False
+
+        self.ui.coverLightIntensity.setText(f'{value}')
+        suc = self.app.cover.lightIntensity(value)
+        if not suc:
+            self.app.message.emit('Light intensity could not be set', 2)
+
         return suc
