@@ -207,8 +207,7 @@ class Device(object):
             self.log.warning(f'Property: {iProperty["propertyType"]} is not Blob')
             return {}
 
-        elementList = iProperty['elementList']
-        return elementList[propertyName]
+        return iProperty['elementList'][propertyName]
 
 
 class Client(QObject):
@@ -922,6 +921,7 @@ class Client(QObject):
                              chunk=chunk,
                              elementList=elementList,
                              defVector=True)
+
         self.signals.newProperty.emit(deviceName, iProperty)
         if isinstance(chunk, indiXML.DefBLOBVector):
             self.signals.defBLOB.emit(deviceName, iProperty)
