@@ -19,13 +19,15 @@
 import logging
 import os
 import json
+import sys
 from queue import Queue
 import platform
 
 # external packages
 from PyQt5.QtCore import QObject, pyqtSignal, QThreadPool, QTimer
-from skyfield.api import Topos
+from skyfield.api import wgs84
 from importlib_metadata import version
+from PyQt5.QtTest import QTest
 
 # local import
 from base.loggerMW import setCustomLoggingLevel
@@ -35,9 +37,9 @@ from logic.powerswitch.kmRelay import KMRelay
 from logic.modeldata.buildpoints import DataPoint
 from logic.modeldata.hipparcos import Hipparcos
 from logic.dome.dome import Dome
-from logic.imaging.camera import Camera
-from logic.imaging.filter import Filter
-from logic.imaging.focuser import Focuser
+from logic.camera.camera import Camera
+from logic.filter.filter import Filter
+from logic.focuser.focuser import Focuser
 from logic.environment.sensorWeather import SensorWeather
 from logic.environment.skymeter import Skymeter
 from logic.environment.onlineWeather import OnlineWeather
@@ -69,6 +71,7 @@ class MountWizzard4(QObject):
     redrawHemisphere = pyqtSignal()
     updatePointMarker = pyqtSignal()
     drawBuildPoints = pyqtSignal()
+    enableEditPoints = pyqtSignal(object)
     drawHorizonPoints = pyqtSignal()
     updateDomeSettings = pyqtSignal()
     sendSatelliteData = pyqtSignal()
@@ -80,11 +83,17 @@ class MountWizzard4(QObject):
     update1s = pyqtSignal()
     update3s = pyqtSignal()
     update10s = pyqtSignal()
+    update30s = pyqtSignal()
     update60s = pyqtSignal()
     update3m = pyqtSignal()
     update10m = pyqtSignal()
     update30m = pyqtSignal()
     update1h = pyqtSignal()
+    start1s = pyqtSignal()
+    start3s = pyqtSignal()
+    start5s = pyqtSignal()
+    start10s = pyqtSignal()
+    start30s = pyqtSignal()
 
     def __init__(self,
                  mwGlob=None,
@@ -100,7 +109,7 @@ class MountWizzard4(QObject):
         self.timerCounter = 0
         self.mainW = None
         self.threadPool = QThreadPool()
-        self.threadPool.setMaxThreadCount(20)
+        self.threadPool.setMaxThreadCount(30)
         self.message.connect(self.writeMessageQueue)
         self.config = {}
         self.loadConfig()
@@ -174,6 +183,9 @@ class MountWizzard4(QObject):
         if os.path.isfile('test.txt'):
             self.update3s.connect(self.quit)
 
+        if len(sys.argv) > 1:
+            self.message.emit(sys.argv[1], 1)
+
     def checkAndSetAutomation(self):
         """
         the windows automation with pywinauto has a serious bug in python lib.
@@ -199,17 +211,17 @@ class MountWizzard4(QObject):
         lat = self.config.get('topoLat', 51.47)
         lon = self.config.get('topoLon', 0)
         elev = self.config.get('topoElev', 46)
-        topo = Topos(longitude_degrees=lon,
-                     latitude_degrees=lat,
-                     elevation_m=elev)
+        topo = wgs84.latlon(longitude_degrees=lon,
+                            latitude_degrees=lat,
+                            elevation_m=elev)
 
         config = self.config.get('mainW', {})
-        if config.get('loglevelDeepDebug', False):
-            level = 'DEBUG'
+        if config.get('loglevelTrace', False):
+            level = 'TRACE'
         elif config.get('loglevelDebug', False):
-            level = 'INFO'
+            level = 'DEBUG'
         else:
-            level = 'WARN'
+            level = 'INFO'
         setCustomLoggingLevel(level)
         return topo
 
@@ -240,6 +252,8 @@ class MountWizzard4(QObject):
             self.update3s.emit()
         if (self.timerCounter + 20) % 100 == 0:
             self.update10s.emit()
+        if (self.timerCounter + 25) % 300 == 0:
+            self.update30s.emit()
         if (self.timerCounter + 25) % 600 == 0:
             self.update60s.emit()
         if (self.timerCounter + 12) % 1800 == 0:
@@ -250,6 +264,16 @@ class MountWizzard4(QObject):
             self.update30m.emit()
         if (self.timerCounter + 15) % 36000 == 0:
             self.update1h.emit()
+        if self.timerCounter == 10:
+            self.start1s.emit()
+        if self.timerCounter == 30:
+            self.start3s.emit()
+        if self.timerCounter == 50:
+            self.start5s.emit()
+        if self.timerCounter == 100:
+            self.start10s.emit()
+        if self.timerCounter == 300:
+            self.start30s.emit()
         return True
 
     def aboutToQuit(self):
@@ -265,7 +289,7 @@ class MountWizzard4(QObject):
         :return:    True for test purpose
         """
         self.aboutToQuit()
-        self.mount.mountUp = False
+        self.deviceStat['mount'] = False
         self.threadPool.waitForDone(5000)
         self.message.emit('MountWizzard4 manual stopped', 1)
         self.application.quit()
@@ -362,13 +386,14 @@ class MountWizzard4(QObject):
         :return:    status how it was called
         """
         if status and not self.mountUp:
+            self.mount.cycleSetting()
             self.mount.getFW()
             self.mount.getLocation()
-            self.mount.cycleSetting()
             self.mainW.refreshName()
             self.mainW.refreshModel()
-            self.mount.getTLE()
             self.mountUp = True
+            QTest.qWait(500)
+            self.mount.getTLE()
             return True
 
         elif not status and self.mountUp:
